
package fr.inria.infovis.panel;

import infovis.Column;
import infovis.column.FilterColumn;
import infovis.panel.DynamicQuery;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.JComponent;
import javax.swing.JTextField;

/**
 * @author raf
 *
 * A replacement of the StringSearchDynamicQuery of Infovis
 * 
 * 
 */
public class StringSearchDynamicQuery extends JTextField implements DynamicQuery {
    protected Column column;
    protected FilterColumn filter;
    protected transient Pattern pattern; 
    
    public StringSearchDynamicQuery(Column col) {
        this.column = col;
        addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String regexp = e.getActionCommand();
                if (regexp.length() == 0)
                    pattern = null;
                else
                    pattern = Pattern.compile(regexp);
                apply();
            }
        });
    }

    public Column getColumn() {
        return column;
    }
    
    public void setFilterColumn(FilterColumn filter) {
        if (this.filter != null) {
            this.filter.removeDynamicQuery(this);
        }
        this.filter = filter;
        if (this.filter != null) {
            this.filter.addDynamicQuery(this);      
        }
    }

    public FilterColumn getFilterColumn() {
        return filter;
    }

    public void apply() {
        if (filter != null)
            filter.applyDynamicQuery(this, column.iterator());
    }


    public boolean isFiltered(int row) {
        if (pattern == null || column.isValueUndefined(row))
            return false;
        String v = column.getValueAt(row);
        Matcher m = pattern.matcher(v);
        return !m.matches();
    }

    

    public JComponent getComponent() {
        return this;
    }

}
