
package fr.inria.infovis.panel.visualization;

import infovis.Graph;
import infovis.column.filter.ComposeExceptFilter;
import infovis.column.filter.ExceptNamed;
import infovis.column.filter.InternalFilter;
import infovis.graph.InDegree;
import infovis.graph.OutDegree;
import infovis.graph.visualization.MatrixVisualPanel;
import infovis.graph.visualization.MatrixVisualization;
import infovis.panel.DetailTable;
import infovis.panel.DynamicQueryPanel;
import infovis.table.FilteredTable;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import fr.inria.infovis.panel.DefaultControlPanel;

/**
 * @author raf
 *
 * A replacement of the MatrixControlPanel of Infovis
 * 
 * 
 */
public class MatrixControlPanel extends DefaultControlPanel {
    protected DetailTable rowDetail;
    protected DetailTable columnDetail;
    protected DynamicQueryPanel rowQueryPanel;
    protected DynamicQueryPanel columnQueryPanel;
    protected FilteredTable filteredGraph;
    protected JCheckBox hideFiltered;
    protected JCheckBox squareEdges;

    /**
     * Constructor for MatrixControlPanel.
     *
     * @param visualization
     */
    public MatrixControlPanel(MatrixVisualization visualization) {
        super(visualization);
    }

    /**
     * Returns the MatrixVisualization.
     *
     * @return the MatrixVisualization.
     */
    public MatrixVisualization getMatrix() {
        return (MatrixVisualization) getVisualization()
            .findVisualization(MatrixVisualization.class);
    }

    /**
     * Returns the Graph
     *
     * @return Return the Graph
     */
    public Graph getGraph() {
        return getMatrix().getGraph();
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public FilteredTable getFilteredGraph() {
        if (filteredGraph == null) {
            filteredGraph =
                new FilteredTable(
                    getGraph().getVertexTable(),
                    new ComposeExceptFilter(
                        new ExceptNamed(OutDegree.OUTDEGREE_COLUMN),
                        new ComposeExceptFilter(
                            new ExceptNamed(InDegree.INDEGREE_COLUMN),
                            InternalFilter.sharedInstance())));
        }
        return filteredGraph;
    }

    /**
     * @see infovis.panel.ControlPanel#createDetailPane()
     */
    protected JComponent createDetailControlPanel() {
        MatrixVisualization matrix = getMatrix();

        Box stack = new Box(BoxLayout.Y_AXIS);
        JComponent det = super.createDetailControlPanel();
        det.setBorder(
            BorderFactory.createTitledBorder("Edge details"));
        stack.add(det);

        rowDetail =
            new DetailTable(
                getFilteredGraph(),
                matrix.getRowSelection());
        JTable rowJTable = new JTable(rowDetail);
        JScrollPane rowJScroll = new JScrollPane(rowJTable);
        rowJScroll.setBorder(
            BorderFactory.createTitledBorder("Row details"));
        stack.add(rowJScroll);

        columnDetail =
            new DetailTable(
                getFilteredGraph(),
                matrix.getColumnSelection());
        JTable columnJTable = new JTable(columnDetail);
        JScrollPane columnJScroll = new JScrollPane(columnJTable);
        columnJScroll.setBorder(
            BorderFactory.createTitledBorder("Column details"));
        stack.add(columnJScroll);
        return stack;
    }

    /**
     * @see infovis.panel.ControlPanel#createFiltersPane()
     */
    protected JComponent createFiltersControlPanel() {
        Box stack = new Box(BoxLayout.Y_AXIS);
        JComponent filters = super.createFiltersControlPanel();
        filters.setBorder(
            BorderFactory.createTitledBorder("Edge dynamic queries"));
        stack.add(filters);

        Box checkBoxes = new Box(BoxLayout.X_AXIS);

        hideFiltered =
            new JCheckBox(
                "Hide Filtereed",
                !getMatrix().isFilteredRowVisible());
        hideFiltered.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                getMatrix().setFilteredColumnVisible(
                    !hideFiltered.isSelected());
                getMatrix().setFilteredRowVisible(
                    !hideFiltered.isSelected());
            }
        });
        checkBoxes.add(hideFiltered);

        squareEdges =
            new JCheckBox("Square Edges", getMatrix().isSquared());
        squareEdges.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                getMatrix().setSquared(squareEdges.isSelected());
            }
        });
        checkBoxes.add(squareEdges);
        stack.add(checkBoxes);

        rowQueryPanel =
            new DynamicQueryPanel(
                getVisualization(),
                getGraph().getVertexTable(),
                getMatrix().getRowFilter(),
                getFilteredGraph().getFilter());
        JScrollPane rowQueryScroll =
            new JScrollPane(
                rowQueryPanel,
                JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
                JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);

        rowQueryScroll.setBorder(
            BorderFactory.createTitledBorder("Row dynamic queries"));
        stack.add(rowQueryScroll);

        columnQueryPanel =
            new DynamicQueryPanel(
                getVisualization(),
                getGraph().getVertexTable(),
                getMatrix().getColumnFilter(),
                getFilteredGraph().getFilter());
        JScrollPane columnQueryScroll =
            new JScrollPane(
                columnQueryPanel,
                JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
                JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        columnQueryScroll.setBorder(
            BorderFactory.createTitledBorder("Column dynamic queries"));
        stack.add(columnQueryScroll);

        return stack;
    }

    /**
     * @see infovis.panel.ControlPanel#createStdVisualPane()
     */
    protected JComponent createVisualPanel() {
        MatrixVisualPanel visualPanel =
            new MatrixVisualPanel(
                getMatrix(),
                getFilteredGraph().getFilter());
        return visualPanel;
    }

}
