/*
 * Created on 12 mai 2004
 *
 */
package fr.inria.infovis.table;

import fr.inria.infovis.column.LayeredColumn;
import infovis.Column;
import infovis.table.DefaultTable;

import java.util.Collection;
import java.util.Vector;

/**
 * @author raf
 *
 * Default implementation of a Layered Table.
 * a table that support insertion of LayeredColumn
 * 
 */
public class DefaultLayeredTable extends DefaultTable implements LayeredTable {
	
	
	/**
	 * <code>layeredColumns</code> stores the references to LayeredColumns in the table
	 * A LayeredTable use mixed columns (layered or not). This collections maitains a references
	 * only with LMayeredColumns and ignore "standard columns"
	 */
	Collection layeredColumns=null;
	
	
	/**
	 * build the collection of layeredColumns 
	 * with a new empty Vector
	 */
	protected void buildLayeredColumns() {
		if (layeredColumns ==null) layeredColumns = new Vector();
	}
	/**
	 * 
	 * Reimplements the standard addColumn method providing
	 * automated add into the LayeredColumns collection 
	 * if the new added columns is a LayeredColumn
	 * 
	 * @see infovis.Table#addColumn(infovis.Column)
	 */
	public void addColumn(Column c) {
		
		if (c instanceof LayeredColumn) {
			buildLayeredColumns();
			layeredColumns.add(c);
			LayeredColumn layer = (LayeredColumn) c;
			super.addColumn(layer.getColumn());
		} else {
			super.addColumn(c);
		}
	}
	/**
	 * @return Returns the layeredColumns.
	 */
	public synchronized Collection getLayeredColumns() {
		return layeredColumns;
	}
	/**
	 * 
	 * This method set the pointer to a new collection.
	 * But this method DOES NOT ADD the columns contained in the collection
	 * to the list of Columns of the Table
	 * 
	 * @param layeredColumns The layeredColumns to set.
	 */
	public synchronized void setLayeredColumns(Collection layeredColumns) {
		this.layeredColumns = layeredColumns;
	}
}
