/*
 * Created on 6 mai 2004
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
package fr.inria.infovis.temporizer;

import infovis.Column;
import infovis.Graph;
import infovis.Table;


/**
 * 
 * Specific Temporizer dedicated to Infovis Graph
 * 
 * @author raf
 *
 */
public class GraphTemporizer extends AbstractTemporizer {

	/**
	 * <code>graph</code> store a reference to the "temporized" graph
	 */
	protected Graph graph;

	
	/**
	 * 
	 * Constructs a new temporizer for a graph
	 * 
	 * @param g graph to be "temporized"
	 */
	public GraphTemporizer(Graph g) {
		this(DEFAULT_DELAY, g);
	}
	
	/**
	 * 
	 * Constructs a new temporizer for a graph with a specific delay
	 * 
	 * @param d The delay in milliseconds
	 * @param g the graph to be temporized
	 */
	public GraphTemporizer(int d, Graph g) {
		super(d,g);
		this.graph = g;
	}
		
	
	
	
	
	/**
	 * 
	 * Disable notification on a Graph
	 * Basically disable notification on Edge and Vertex tables.
	 * 
	 * @see fr.inria.infovis.temporizer.AbstractTemporizer#disableNotify()
	 */
	protected void disableNotify() {
		Table nodeTable = graph.getVertexTable();
		Table edgeTable = graph.getEdgeTable();
		Column c;
		for(int i=0; i < nodeTable.getColumnCount(); i++) {
			c=nodeTable.getColumnAt(i);
			c.disableNotify();
		}
		
		for(int i=0; i < edgeTable.getColumnCount(); i++) {
			c=edgeTable.getColumnAt(i);
			c.disableNotify();
		}
	}
	/**
	 * 
	 * Enable notification on a Graph
	 * Basically enable notification on Edge and Vertex tables.
	 * 
	 * @see fr.inria.infovis.temporizer.AbstractTemporizer#disableNotify()
	 */
	protected void enableNotify() {
		Table nodeTable = graph.getVertexTable();
		Table edgeTable = graph.getEdgeTable();
		Column c;
		for(int i=0; i < nodeTable.getColumnCount(); i++) {
			c=nodeTable.getColumnAt(i);
			try {
				c.enableNotify();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		
		for(int i=0; i < edgeTable.getColumnCount(); i++) {
			c=edgeTable.getColumnAt(i);
			try {
				c.enableNotify();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
	
}
