/*
 * Created on 30 oct. 2003
 */
package fr.inria.oadymppac.io;
import java.io.File;
import java.io.InputStream;
import java.util.Collection;
import javax.swing.JFrame;
import javax.swing.ProgressMonitorInputStream;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import fr.inria.oadymppac.application.ApplicationProperties;
import fr.inria.oadymppac.panel.PanelFactory;
import fr.inria.oadymppac.parsing.ContentHandler;
import fr.inria.oadymppac.parsing.event.EventListener;
import fr.inria.oadymppac.parsing.gentra4cp.handlers.DefaultValidationErrorHandler;
import fr.inria.oadymppac.parsing.gentra4cp.handlers.TraceContentHandler;
import fr.inria.oadymppac.solver.parsing.SingleComplementEventListener;
import fr.inria.oadymppac.solver.parsing.SingleControlEventListener;
import fr.inria.oadymppac.solver.parsing.SingleHeadEventListener;
import fr.inria.oadymppac.solver.parsing.SingleProvideEventListener;
import fr.inria.oadymppac.solver.parsing.SingleSolverEventListener;
import fr.inria.oadymppac.utils.DefaultProperties;
import fr.inria.oadymppac.utils.Properties;
import fr.inria.oadymppac.widget.SwingWorker;
/**
 * 
 * A Thread class to load file in the application It us in fact the swingWorker
 * specific thread implementation
 * 
 * @author raf
 *  
 */
public class FileLoaderThread extends SwingWorker implements Properties {
	/**
	 * <code>properties</code> The property table containing information
	 * useful for the thread comming from its caller
	 */
	protected Properties properties;
	/**
	 * <code>file</code> The GENTRA4CP file to parse
	 */
	protected File file;
	/**
	 * <code>parser</code> The JAXP SAX parser used to parse the XML document
	 */
	protected SAXParser parser;
	/**
	 * <code>reader</code> The JAXP SAX reader used to parse the XML document
	 */
	protected XMLReader reader;
	/**
	 * <code>defaultHandler</code> DefaultHandler to manage Entity and DTD
	 */
	protected DefaultHandler defaultHandler;
	/**
	 * <code>errorHandler</code> An ErrorHandler, used only to display
	 * validation error on the console 
	 */
	protected ErrorHandler errorHandler;
	/**
	 * <code>mainHandler</code> A Content Handler to manage the content of the
	 * XML document
	 */
	protected ContentHandler mainHandler;
	/**
	 * <code>headListener</code> a delegate event listener to handle the
	 * header section of the XML document
	 *  
	 */
	protected SingleHeadEventListener headListener;
	/**
	 * <code>providableListener</code> A ProvideEventListener to handle the
	 * provide of the header of the XML document
	 */
	protected SingleProvideEventListener providableListener;
	/**
	 * <code>providedListener</code> A ProvideEventListener to handle the
	 * provide of the trace in the XML document
	 */
	protected SingleProvideEventListener providedListener;
	/**
	 * <code>controlListener</code> A Control listener to manage delegated
	 * packet and breakpoint
	 */
	protected SingleControlEventListener controlListener;
	/**
	 * <code>complementListener</code> A control listener to manage complement
	 * events in the XML document
	 */
	protected SingleComplementEventListener complementListener;
	/**
	 * <code>solverListener</code> The main event listener used to handle
	 * trace events
	 */
	protected EventListener solverListener;
	/**
	 * <code>interrupted</code> Indicates the user interrupt manually the parsing
	 * using the cancel button of the progress monitor
	 */
	protected boolean interrupted = false;
	public FileLoaderThread(Properties properties, File file) {
		super();
		this.properties = new DefaultProperties(properties);
		this.file = file;
	}
	/**
	 * 
	 * Construct a strucutre of the parsed document
	 * 
	 * @see fr.emn.oadymppac.widgets.SwingWorker#construct()
	 */
	public Object construct() {
		PanelFactory panelFactory = (PanelFactory) properties
				.getProperty(ApplicationProperties.PANEL_FACTORY);
		JFrame frame = (JFrame) properties
				.getProperty(ApplicationProperties.MAIN_FRAME);
		String title = frame.getTitle();
		try {
			setInterrupted(false); // managing potential user's cancellation of
			// the thread
			InputSource is = new PostMortemInputSource(file);// declare a
			// postmorterm
			// inputsource
			InputStream pgs = is.getByteStream(); // get the inputStream used by
			// the ProgressMonitor
			InputStream progress = new ProgressMonitorInputStream(frame,
					"Reading " + file, pgs); // create the progress monitor
			is.setByteStream(progress); // set the parsed inputsource wth the
			// progress bar
			SAXParserFactory factory = SAXParserFactory.newInstance(); // use
			// the
			// SAX
			// Framework
			// to get
			// a
			// SAXParserFactory
			factory.setValidating(true); // set validation as possible (means
			// evetually performed)
			parser = factory.newSAXParser(); // create a parser using the
			// factory
			defaultHandler = new DefaultHandler(); // create the default handler
			// for error
			reader = parser.getXMLReader(); // create a reader using the factory
			init_handlers(); // init the Content handler and event listeners
			reader.setContentHandler(mainHandler);
			reader.setDTDHandler(defaultHandler);
			reader.setEntityResolver(defaultHandler);
			//check if validation is required
			String validation = (String) properties
					.getProperty(ApplicationProperties.XML_VALIDATION);
			if (validation != null && validation.equals("true")) {
				errorHandler = new DefaultValidationErrorHandler();
			} else {
				errorHandler = defaultHandler;
			}
			reader.setErrorHandler(errorHandler);
			reader.parse(is); // parse the document
			if (errorHandler instanceof DefaultValidationErrorHandler) {
				Collection exceptions = ((DefaultValidationErrorHandler) errorHandler)
						.getExceptions();
				if (exceptions != null && exceptions.size() > 0) {
					panelFactory.buildErrorPanel(new Exception(
							"Parsing ended with " + exceptions.size()
									+ " parsing error(s)"));
				}
			}
			frame.setTitle(title + " " + file);
		} catch (java.io.InterruptedIOException interrupt) {
			// this specific Exception results form user's manual interruption
			panelFactory.buildMessagePanel("Interrupted by user!");
			frame.setTitle(title + " " + file);
			setInterrupted(true);
		} catch (Exception e) {
			//Results from real Exception (mainly invalid XML document)
			panelFactory.buildErrorPanel(e);
		}
		return solverListener; // return the synchronized structure of the document
	}
	/**
	 * 
	 * set up handler and event listeners relation
	 * 
	 */
	protected void init_handlers() {
		mainHandler = (mainHandler == null ? buildMainHandler() : mainHandler);
		headListener = (headListener == null
				? new SingleHeadEventListener()
				: headListener);
		providableListener = (providableListener == null
				? new SingleProvideEventListener(
						SingleProvideEventListener.HEAD)
				: providableListener);
		providedListener = (providedListener == null
				? new SingleProvideEventListener(
						SingleProvideEventListener.TRACE)
				: providedListener);
		controlListener = (controlListener == null
				? new SingleControlEventListener()
				: controlListener);
		complementListener = (complementListener == null
				? new SingleComplementEventListener()
				: complementListener);
		solverListener = (solverListener == null
				? new SingleSolverEventListener()
				: solverListener);
		initMainHandler();
	}
	
	
	
	/**
	 * This version of the thread uses a first version of the TraceContnetHandler
	 * 
	 * @return a new instance of a TracEHandler
	 */
	protected ContentHandler buildMainHandler() {
		return new TraceContentHandler();
	}
	/**
	 * 
	 * In it specific configuration of the main trace ContentHandler
	 * 
	 */
	protected void initMainHandler() {
		TraceContentHandler handler = (TraceContentHandler) mainHandler;
		handler.addControlListener(controlListener);
		handler.addHeadListener(headListener);
		handler.addProvidableListener(providableListener);
		handler.addProvidedListener(providedListener);
		handler.addComplementListener(complementListener);
	}
	/**
	 * 
	 * Set the complemnt Event listener for the thread
	 * 
	 * @param complementListener
	 *            The complementListener to set.
	 */
	public void setComplementListener(
			SingleComplementEventListener complementListener) {
		this.complementListener = complementListener;
	}
	/**
	 * Set the control Event listener for the thread
	 * 
	 * @param controlListener
	 *            The controlListener to set.
	 */
	public void setControlListener(SingleControlEventListener controlListener) {
		this.controlListener = controlListener;
	}
	/**
	 * 
	 * Set the header Event listener for the thread
	 * 
	 * 
	 * @param headListener
	 *            The headListener to set.
	 */
	public void setHeadListener(SingleHeadEventListener headListener) {
		this.headListener = headListener;
	}
	/**
	 * 
	 * Set the Header provide Event listener for the thread
	 * 
	 * @param providableListener
	 *            The providableListener to set.
	 */
	public void setProvidableListener(
			SingleProvideEventListener providableListener) {
		this.providableListener = providableListener;
	}
	/**
	 * 
	 * Set the trace provide Event listener for the thread
	 * 
	 * @param providedListener
	 *            The providedListener to set.
	 */
	public void setProvidedListener(SingleProvideEventListener providedListener) {
		this.providedListener = providedListener;
	}
	/**
	 * 
	 * Set the trace Event listener for the thread
	 * 
	 * 
	 * @param solverListener
	 *            The solverListener to set.
	 */
	public void setSolverListener(EventListener solverListener) {
		this.solverListener = solverListener;
		if (solverListener != null)
			addListener(solverListener);
	}
	/**
	 * Add another trace event Listener
	 * 
	 * @param solverListener
	 */
	public void addListener(EventListener listener) {
		if (mainHandler == null)
			init_handlers();
		((TraceContentHandler) mainHandler).addDefaultListener(listener);
	}
	/**
	 * @return Returns the properties.
	 */
	public Properties getProperties() {
		return properties;
	}
	/**
	 * Ste the property list used to configure the thread
	 * 
	 * @param properties
	 *            The properties to set.
	 */
	public void setProperties(Properties properties) {
		this.properties = properties;
	}
	/**
	 * 
	 * Get a property form the property list used to configure the thread
	 * 
	 * @param property
	 * @return
	 */
	public Object getProperty(String property) {
		return properties.getProperty(property);
	}
	/**
	 * 
	 * Set/add a property inside the table used to configure the thread
	 * 
	 * @param property
	 * @param value
	 */
	public void setProperty(String property, Object value) {
		properties.setProperty(property, value);
	}
	/**
	 * @return Returns if th user has interruptedthe parser
	 */
	public synchronized boolean isInterrupted() {
		return interrupted;
	}
	/**
	 * 
	 * Indicates that the user stopped manually the parsing process (using the cancel button of the Progress Monitor)
	 * 
	 * @param interrupted
	 *            The interrupted to set.
	 */
	public synchronized void setInterrupted(boolean interrupted) {
		this.interrupted = interrupted;
	}
}