/*
 * Created on 6 avr. 2004
 * 
 * To change the template for this generated file go to Window - Preferences -
 * Java - Code Generation - Code and Comments
 */
package fr.inria.oadymppac.io;
import infovis.Visualization;

import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.JFrame;

import fr.inria.infovis.temporizer.AbstractTemporizer;
import fr.inria.oadymppac.application.ApplicationProperties;
import fr.inria.oadymppac.panel.ApplicationPanelFactory;
import fr.inria.oadymppac.panel.JListDialog;
import fr.inria.oadymppac.panel.VisualizationFrameDecorator;
import fr.inria.oadymppac.solver.DefaultSolver;
import fr.inria.oadymppac.solver.Solver;
import fr.inria.oadymppac.solver.analyser.AbstractGraphAnalyser;
import fr.inria.oadymppac.solver.analyser.ConstraintConstraintGraphAnalyser;
import fr.inria.oadymppac.solver.analyser.VariableConstraintGraphAnalyser;
import fr.inria.oadymppac.solver.parsing.ConstraintConstraintManagerEventListener;
import fr.inria.oadymppac.solver.parsing.SolverManagerEventListener;
import fr.inria.oadymppac.solver.parsing.VariableConstraintManagerEventListener;
import fr.inria.oadymppac.utils.Properties;
/**
 * 
 * A Concrete implementation of a file manager for the application
 * 
 * @author raf
 *
 */
public class GenericTraceFileManager implements FileManager {
	/**
	 * <code>properties</code> The property table shared with the application 
	 */
	Properties properties;
	/**
	 * <code>fileChooser</code> The file chooser used when choosing a file
	 */
	JFileChooser fileChooser = null;
	/**
	 * <code>filter</code> The filter provided to the FileChooser
	 */
	GenericTraceFileFilter filter = null;
	/**
	 * 
	 * Defaulmt constructor
	 * 
	 * @param application : the properties shared with the application
	 */
	public GenericTraceFileManager(Properties application) {
		this.properties = application;
	}
	/**
	 * 
	 * Select a file using the JFileChooser
	 * 
	 * @see fr.inria.oadymppac.io.FileManager#selectFile()
	 */
	public File selectFile() {
		if (filter == null) {
			filter = new GenericTraceFileFilter();
		}
		if (fileChooser == null) {
			fileChooser = new JFileChooser();
			fileChooser
					.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
			fileChooser.setCurrentDirectory((DEFAULT_FOLDER.exists()
					? DEFAULT_FOLDER
					: new File(".")));
			fileChooser.setFileFilter(filter);
			fileChooser.setDoubleBuffered(false);
		}
		int ret = fileChooser.showOpenDialog(null);
		if (ret == JFileChooser.APPROVE_OPTION) {
			return fileChooser.getSelectedFile();
		} else
			return null;
	}
	/**
	 * 
	 * Load a file 
	 * 
	 * @see fr.inria.oadymppac.io.FileManager#loadFile(java.io.File)
	 */
	public void loadFile(File file) {
		reset(); // resetting previously loaded file
		System.out.println("File Loading started");
		long time=System.currentTimeMillis();
		ApplicationPanelFactory panelFactory = (ApplicationPanelFactory) properties
				.getProperty(ApplicationProperties.PANEL_FACTORY); // get The panel Factory
		Solver solver = new DefaultSolver(properties); // Create a solver (main strucuration of the trace
		SolverManagerEventListener solverManager = new SolverManagerEventListener(
				solver); // create the Eventlistener in charge to structure the parsed trace
		ConstraintConstraintManagerEventListener constraintManager = null; // This event listener build the constraintcontraint graph analysis
		VariableConstraintManagerEventListener variableManager = null;// This event listener build the varaiblecontraint graph analysis
		JFrame frame = (JFrame) properties.getProperty(ApplicationProperties.MAIN_FRAME);
		Object[] analysers = JListDialog.getSelection(); // get the selected analysis chosen by the user
		Visualization[] visualizations = (analysers == null ? null : new Visualization[analysers.length]);
		
		FasterFileLoaderThread loader = new FasterFileLoaderThread(properties, file); // build a new Loading thread
		loader.setSolverListener(solverManager);
		
		
		AbstractGraphAnalyser analyser=null;
		AbstractTemporizer temporizer=null;
		// build the analysers and visualizations correcponding to user's choices 
		for (int i = 0; analysers != null && i < analysers.length; i++) {
			if (analysers[i] == ConstraintConstraintGraphAnalyser.LABEL) {
				constraintManager = new ConstraintConstraintManagerEventListener(
						solver);
				visualizations[i] = panelFactory.buildVisualization(constraintManager.getGraph());
				analyser = constraintManager.getAnalyser();
				temporizer = (analyser == null ? null : analyser.getTemporizer());
				if (temporizer != null) temporizer.setVisualization(visualizations[i]);
				loader.addListener(constraintManager);
				analyser=null;
				temporizer=null;
			}
			if (analysers[i] == VariableConstraintGraphAnalyser.LABEL) {
				variableManager = new VariableConstraintManagerEventListener(
						solver);
				visualizations[i] = panelFactory.buildVisualization(variableManager.getGraph());
				analyser = variableManager.getAnalyser();
				temporizer = (analyser == null ? null : analyser.getTemporizer());
				if (temporizer != null) temporizer.setVisualization(visualizations[i]);
				loader.addListener(variableManager);
				analyser=null;
				temporizer=null;				
			}
		}
		
		// launch the loading thread
		solverManager = (SolverManagerEventListener) loader.construct();
		for (int i=0; i <10; i++) System.gc(); // should make a big cleaning stuff !
		if (loader.isInterrupted()) {
			if (solverManager != null) {
				solverManager.processEnd(); // should terminate properly the analysis.
			}
			if (constraintManager != null) {
				constraintManager.processEnd(); // should terminate properly the analysis.
			}
			if (variableManager != null) {
				variableManager.processEnd(); // should terminate properly the analysis.
			}			
		}
		properties.setProperty(ApplicationProperties.CURRENT_DOCUMENT,
					solver);
		VisualizationFrameDecorator frameDecorator = (VisualizationFrameDecorator) properties
					.getProperty(ApplicationProperties.VISUALIZATION_FRAME_DECORATOR);
		frameDecorator.enableAnalyseMenu();
		System.out.println("File Loading terminated:"+ (System.currentTimeMillis()-time)+" ms");
		
	}
	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.oadymppac.io.FileManager#saveFile(java.io.File)
	 */
	public void saveFile(File file) {
	}
	
	
	/**
	 * Reset a previoulsy loaded file
	 * 
	 * 
	 */
	protected void reset() {
		Solver doc = (Solver) properties.getProperty(ApplicationProperties.CURRENT_DOCUMENT);
		if (doc != null) { 
			doc.reset();
			properties.setProperty(ApplicationProperties.CURRENT_DOCUMENT,
				new DefaultSolver(properties));
		}
		VisualizationFrameDecorator frameDecorator = (VisualizationFrameDecorator) properties
		.getProperty(ApplicationProperties.VISUALIZATION_FRAME_DECORATOR);
		frameDecorator.disableAnalyseMenu();

	}
}