/*
 * INRIA-OADYMPPAC
 * 
 * Ownership: Copyright (c) 1990-2004 COSYTEC S.A.
 * Author	: 	raf
 *           	COSYTEC
 *		    	4 rue Jean Rostand
 * 				91893 Orsay Cedex, France
 *				Tel. +33 1 60 19 37 38
 *				Fax. +33 1 60 19 36 20
 *
 * Purpose	: COMPLETE HERE
 * 
 * Created on 17 mars 2004
 *
 * 
 */
package fr.inria.oadymppac.io;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.zip.GZIPInputStream;


/**
 * 
 * An implementation of GENTRACP Input source 
 * using local files
 * 
 * @author raf
 *
 */
public class PostMortemInputSource extends org.xml.sax.InputSource implements GenericTraceInputSource {
 
    /**
     * 
     * Construct a new input source using a File object
     * Handles automaitically compressed/non compressed files
     * 
     * @param file The file used as input source 
     * @throws FileNotFoundException  exception if files does not exists 
     * @throws IOException I/O error on the give file
     */
    public PostMortemInputSource(File file) throws FileNotFoundException, IOException {
        this(file.getAbsoluteFile().getAbsolutePath());
    }
    
    /**
     * Construct a new input source using a String as filename
     * Handles automaitically compressed/non compressed files
     * 
     * @param file The file used as input source 
     * @throws FileNotFoundException  exception if files does not exists 
     * @throws IOException I/O error on the give file
     */
    public PostMortemInputSource(String filename) throws FileNotFoundException, IOException {
        this(new URL("file:"+filename));
    }
        
    /**
    * 
    *Construct a new input source using a URL object
    * Handles automaitically compressed/non compressed files
    * 
    * @param file The file used as input source 
    * @throws FileNotFoundException  exception if files does not exists 
    * @throws IOException I/O error on the give file    
    * 
    */
    public PostMortemInputSource(URL url ) throws FileNotFoundException, IOException {
        InputStream is;
        BufferedInputStream bis;
        if (url.getFile().endsWith(GenericTraceFileFilter.COMPRESSED_EXT)) {
            is = new GZIPInputStream(new FileInputStream(url.getFile())); 
        } else {
            is = new FileInputStream(url.getFile());
        }    
        bis = new BufferedInputStream(is);
        this.setByteStream(bis);
        this.setSystemId(url.toExternalForm());
        this.setPublicId(url.getFile());
    }

}
