/*
 * Created on 6 avr. 2004
 * 
 * To change the template for this generated file go to Window - Preferences -
 * Java - Code Generation - Code and Comments
 */
package fr.inria.oadymppac.panel;
import infovis.Graph;
import infovis.Visualization;
import fr.inria.infovis.panel.SplashPanel;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import org.apache.log4j.Logger;
import fr.inria.oadymppac.application.ApplicationProperties;
import fr.inria.oadymppac.application.MainApplication;
import fr.inria.oadymppac.solver.analyser.SolverAnalyser;
import fr.inria.oadymppac.utils.Properties;
/**
 * 
 * A concrete implementation of an Application Panel Factory
 * 
 * @author raf
 *  
 */
public class ApplicationPanelFactory implements PanelFactory {
	/**
	 * <code>properties</code> Proerty table shared with the application
	 */
	Properties properties;
	/**
	 * <code>fileChooser</code> The JFileChooser used by the application
	 */
	protected JFileChooser fileChooser = null;
	/**
	 * <code>splashScreen</code> The Splash screen of the applciation
	 */
	protected JComponent splashScreen = null;
	/**
	 * <code>mainFrame</code> The main Frame of the application
	 */
	protected JFrame mainFrame = null;
	public ApplicationPanelFactory(Properties properties) {
		this.properties = properties;
	}
	/**
	 * 
	 * Build an Error Message box using an Exception
	 * 
	 * @see fr.inria.oadymppac.panel.PanelFactory#buildErrorPanel(java.lang.Exception)
	 */
	public void buildErrorPanel(Exception e) {
		JFrame frame = (JFrame) properties
				.getProperty(ApplicationProperties.MAIN_FRAME);
		JOptionPane.showMessageDialog(frame, e.getMessage(), "Exception...",
				JOptionPane.ERROR_MESSAGE);
		Logger logger = (Logger) properties
				.getProperty(ApplicationProperties.LOGGER);
		if (logger != null && logger.isDebugEnabled())
			logger.error("Exception thrown:", e);
		else
			e.printStackTrace();
	}
	/**
	 * Build an Error Message box using an Error
	 * 
	 * @see fr.inria.oadymppac.panel.PanelFactory#buildErrorPanel(java.lang.Error)
	 */
	public void buildErrorPanel(Error e) {
		JFrame frame = (JFrame) properties
				.getProperty(ApplicationProperties.MAIN_FRAME);
		JOptionPane.showMessageDialog(frame, e.getMessage(), "Error...",
				JOptionPane.ERROR_MESSAGE);
		Logger logger = (Logger) properties
				.getProperty(ApplicationProperties.LOGGER);
		if (logger != null && logger.isDebugEnabled())
			logger.error("Error occured:", e);
		else
			e.printStackTrace();
	}
	/**
	 * 
	 *Display a message in a message box
	 * 
	 *  
	 * @see fr.inria.oadymppac.panel.PanelFactory#buildErrorPanel(java.lang.Exception)
	 */
	public void buildMessagePanel(String message) {
		JFrame frame = (JFrame) properties
				.getProperty(ApplicationProperties.MAIN_FRAME);
		JOptionPane.showMessageDialog(frame, message, "Information...",
				JOptionPane.INFORMATION_MESSAGE);
		Logger logger = (Logger) properties
				.getProperty(ApplicationProperties.LOGGER);
	}
	/**
	 *
	 * Display the about panel
	 * 
	 * @see fr.inria.oadymppac.panel.PanelFactory#buildErrorPanel(java.lang.Exception)
	 */
	public void buildAboutPanel() {
		JFrame frame = (JFrame) properties
				.getProperty(ApplicationProperties.MAIN_FRAME);
		MainApplication app = (MainApplication) properties
				.getProperty(ApplicationProperties.APPLICATION);
		JOptionPane.showMessageDialog(frame, "The OADYMPPAC Trace Analyzer V"
				+ app.getMajorVersion() + "." + app.getMinorVersion() + "."
				+ +app.getBuildVersion() + "\n" + "A collaboration between:\n "
				+ "\tCosytec S.A. ,France\n"
				+ "\tEMN (Ecole des Mines de Nantes), France\n"
				+ "\tILOG S.A. ,France\n" + "\tINRIA-Contraintes, France\n"
				+ "\tINRIA-Futurs, France\n" + "\tINSA-Rennes, France\n"
				+ "\tLIFO (Universit d'Orlans), France", "About...",
				JOptionPane.INFORMATION_MESSAGE);
	}
	/** 
	 * Display a file chooser
	 *  @see fr.inria.oadymppac.panel.PanelFactory#buildFileSelector()
	 */
	public JFileChooser buildFileSelector() {
		fileChooser = (fileChooser == null ? new JFileChooser() : fileChooser);
		return fileChooser;
	}
	/** 
	 * Display a splash panel
	 * @see fr.inria.oadymppac.panel.PanelFactory#buildSplashScreen()
	 */
	public JComponent buildSplashScreen() {
		splashScreen = (splashScreen == null ? new SplashPanel() : splashScreen);
		splashScreen.setOpaque(true);
		return splashScreen;
	}
	
	
	/**
	 * Display a selector of analysers 
	 * 
	 * @return The cancel or OK user selection
	 */
	public int buildAnalysisSelector() {
		JFrame frame = (JFrame) properties
				.getProperty(ApplicationProperties.MAIN_FRAME);
		return JListDialog.showDialog(frame, SolverAnalyser.LABELS, null);
	}
	/**
	 * 
	 * Display a Question in a Message box
	 * 
	 * @see fr.inria.oadymppac.panel.PanelFactory#buildQuestionPanel(java.lang.String)
	 */
	public int buildQuestionPanel(String question) {
		JFrame frame = (JFrame) properties
				.getProperty(ApplicationProperties.MAIN_FRAME);
		return JOptionPane.showConfirmDialog(frame, question, "Question...",
				JOptionPane.YES_NO_OPTION);
	}
	/**
	 * 
	 * Uninstall a view from the main workspace
	 */
	protected void uninstallView() {
		System.out.println("UNINSTALLING VIEW");
		JFrame frame = (JFrame) properties
				.getProperty(ApplicationProperties.MAIN_FRAME);
		if (frame.getContentPane() != null && splashScreen != null) {
			frame.getContentPane().remove(splashScreen);
		}
	}
	/**
	 *
	 *Build a new visualization (INFOVIS) from a Graph
	 * 
	 * @param graph
	 * @return the visualization created
	 */
	public Visualization buildVisualization(Graph graph) {
		uninstallView();
		System.out.println("Build visualization with graph " + graph);
		VisualizationFrameDecorator frameDeco = (VisualizationFrameDecorator) properties
				.getProperty(ApplicationProperties.VISUALIZATION_FRAME_DECORATOR);
		return (frameDeco == null ? null : frameDeco
				.createGraphVisualization(graph));
	}
	/**
	 * @return Returns the fileChooser.
	 */
	public synchronized JFileChooser getFileChooser() {
		if (fileChooser == null)
			fileChooser = new JFileChooser();
		return fileChooser;
	}
	/**
	 * @return Returns the splashScreen.
	 */
	public synchronized JComponent getSplashScreen() {
		if (splashScreen == null)
			splashScreen = new SplashPanel();
		return splashScreen;
	}
	/**
	 * Build the main Frame of the application
	 * 
	 * @see fr.inria.oadymppac.panel.PanelFactory#buildMainFrame()
	 */
	public JFrame buildMainFrame() {
		if (mainFrame == null) {
			mainFrame = (JFrame) properties
					.getProperty(ApplicationProperties.MAIN_FRAME);
		}
		return mainFrame;
	}
}