/*
 * Created on 20 avr. 2004
 *
 */
package fr.inria.oadymppac.panel;

import fr.inria.infovis.column.LayeredColumn;
import infovis.panel.DoubleBoundedRangeModel;
import infovis.panel.DoubleRangeSlider;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.util.Collection;
import java.util.Iterator;
import java.util.Timer;
import java.util.TimerTask;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JToggleButton;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * 
 * A specicif component susited to control the history system
 * 
 * @author raf
 *
 */
public class HistoryController extends JPanel implements ChangeListener, ActionListener {
	/**
	 * <code>historyFwd</code> The forward button
	 */
	JButton historyFwd;
    /**
     *<code>historyRwd</code> The Rewind button of the history
     */
    JButton historyRwd;
    /**
     * <code>historySlider</code> The slider to adjust the hisotry position
     */
    DoubleRangeSlider historySlider;
    /**
     * <code>playStep</code> The textfiled to get the anaimation speed
     */
    JTextField playStep;
    /**
     * <code>playStep</code> The button to play an animated view
     */
    JToggleButton playStop;
    
    /**
     * <code>model</code> The model of the component
     */
    HistoryModel model;
    
    /**
     * <code>historyTimer</code> The timer for animation
     */
    Timer historyTimer;
    /**
     * <code>anim</code> The timertask performing the animation
     */
    HistoryTimerTask anim; 
    
    /**
     *  <code>playing</code> Indicates if an animation is currenlty playing
     */
    boolean playing = false;
    
    /**
     * <code>animExtent</code> Size of the knob of the slider
     */
    double animExtent;
    /**
     * <code>animValue</code> Position of the knob of the slider
     */
    double animValue;
    
    public HistoryController() {
    	super();
    	init();
    }
    
    
    /**
     * Ensure that the size of the component will be a fixed height
     * 
     * @see java.awt.Component#getPreferredSize()
     */
    public Dimension getPreferredSize() {
    	return new Dimension(getWidth(), 24);
    }
    
    /**
     * Init the component
     */
    protected void init() {
    	setLayout(new BorderLayout());
    	
    	model = new HistoryModel(0.0,1000.0,0.0,1000.0);
    	historyFwd =new JButton(">>") ;
    	historySlider = new DoubleRangeSlider(model) {
    		/* (non-Javadoc)
    		 * @see javax.swing.JComponent#getToolTipText(java.awt.event.MouseEvent)
    		 */
    		public String getToolTipText(MouseEvent event) {
    			return "" 
    			    +"FROM:"+Math.round(getLowValue())+", "
    			    +"TO:"+Math.round(getHighValue())+", "
					+"MIN:"+Math.round(getMinimum())+", "
    			    +"MAX:"+Math.round(getMaximum());
    		    }
    	};
    	JPanel playPanel = new JPanel(new BorderLayout()); 
    	historyRwd =new JButton("<<") ;
    	playStep = new JTextField("1") {
    		public Dimension getPreferredSize() {
    			return new Dimension(40, 10);
    		}
   	};
    	playStop = new JToggleButton(new ImageIcon("images/right.gif")) {
    		public Dimension getPreferredSize() {
    			return new Dimension(24, 24);
    		}
    	}; 
    	add(historyFwd, BorderLayout.EAST);
    	add(historySlider, BorderLayout.CENTER);
    	playPanel.add(historyRwd, BorderLayout.EAST);
    	playPanel.add(playStep,BorderLayout.CENTER);
    	playPanel.add(playStop,BorderLayout.WEST);
    	add(playPanel,BorderLayout.WEST);
    	historySlider.setEnabled(true);
    	addChangeListener(this);
    	historyFwd.addActionListener(this);
    	historyRwd.addActionListener(this);
    	playStop.addActionListener(this);
    	historyTimer = new Timer();
    	
    	
    }
    
    
    
	
	/**
	 * Update the model => perform refresh
	 */
	protected void updateModel() {		
		Collection columns = model.getColumns();
		Iterator it = (columns == null ? null : columns.iterator());
		LayeredColumn column;
		double from =  model.getValue();
		double to = from + model.getExtent();
		while(it != null && it.hasNext()) {
			column = (LayeredColumn) it.next(); 		
			column.restoreLayers((int) from, (int) to );
		}
	}

	/**
	 * Set the maximum value for the history
	 * 
	 * @param max the maximum to set
	 */
	public void setMaximum(double max) {
		historySlider.setMaximum(max);
	}
	/**
	 * Set the minimum value for the history
	 * 
	 * @param min the minimum to set
	 */
	public void setMinimum(double min) {
		historySlider.setMinimum(min);
	}
	
	/**
	 * Add a change listener when adjusting the slider
	 * 
	 * @param l The listener tracking adjustments
	 */
	public void addChangeListener(ChangeListener l) {
		DoubleBoundedRangeModel md = historySlider.getModel();
		md.addChangeListener(l);
	}


	/**
	 * Add a state listener when adjusting the slider
	 * 
	 * @see javax.swing.event.ChangeListener#stateChanged(javax.swing.event.ChangeEvent)
	 */
	public void stateChanged(ChangeEvent e) {		
		DoubleBoundedRangeModel rmd = (DoubleBoundedRangeModel) e.getSource();
		if (! rmd.getValueIsAdjusting()) {
			rmd.setExtent(Math.rint(rmd.getExtent()));
			rmd.setValue(Math.rint(rmd.getValue()));
			updateModel();
		}
	}
	
	
	
	/**
	 * @return Returns the model.
	 */
	public synchronized HistoryModel getModel() {
		return model;
	}
	/**
	 * @param model The model to set.
	 */
	public synchronized void setModel(HistoryModel model) {
		this.model = model;
	}

	/**
	 * 
	 * Handle buttons actions
	 * 
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		Object source = e.getSource();
		if (source == historyFwd || source == historyRwd) {
			DoubleBoundedRangeModel rmd = (DoubleBoundedRangeModel) historySlider.getModel();
			double inc = Math.max(1.0,rmd.getExtent() /2);
			double value = rmd.getValue();
			if (e.getSource() == historyFwd) {
				rmd.setValue(Math.min(value+inc, rmd.getMaximum()));
			} else if (e.getSource() == historyRwd) {	
				rmd.setValue(Math.max(value-inc, rmd.getMinimum()));
			}
			historySlider.setModel(rmd);
			updateModel();
		} else if (source == playStop) {
			if (playing) {
				anim.cancel();
				model.setValue(animValue);
				model.setExtent(animExtent);
				historySlider.setModel(model);
				updateModel();
				historyFwd.setEnabled(true);
				historyRwd.setEnabled(true);
				historySlider.setEnabled(true);

			} else {
				historyFwd.setEnabled(false);
				historyRwd.setEnabled(false);
				historySlider.setEnabled(false);
				anim=new HistoryTimerTask(this);
				animExtent = model.getExtent();
				animValue = model.getValue();
				historyTimer.schedule(anim,500);
			}
			playing = ! playing;
		}
	}
	
	/**
	 * 
	 * Inner class of the aniamtion timer task
	 * 
	 * @author raf
	 */
	protected class HistoryTimerTask extends TimerTask {
		/**
		 * <code>historyController</code> The historyController component associated to the anaimation task
		 */
		protected HistoryController historyController;
		/**
		 * Cancel the animation
		 * 
		 * @see java.util.TimerTask#cancel()
		 */
		public boolean cancel() {
			return super.cancel();
		}
		/**
		 * Run the anaimation
		 * 
		 * @see java.lang.Runnable#run()
		 */
		public void run() {
//			if (System.currentTimeMillis() - scheduledExecutionTime() <= 500) {
				animate();
				anim = new HistoryTimerTask(historyController);
				historyTimer.schedule(anim,500);
//			}
		}
		/**
		 * 
		 * Schedule the next step of the animation
		 * 
		 * @see java.util.TimerTask#scheduledExecutionTime()
		 */
		public long scheduledExecutionTime() {
			return super.scheduledExecutionTime();
		}
		/**
		 * 
		 * default Constructor
		 * 
		 */
		public HistoryTimerTask(HistoryController panel) {
			super();
			this.historyController = panel;
		}
		
		/**
		 * Perform an animation step
		 */
		protected void animate() {
			double extent,value,min,max;
			DoubleBoundedRangeModel rmd = historySlider.getModel();
			min = rmd.getMinimum();
			max =rmd.getMaximum();
			try {
				extent = Double.parseDouble(playStep.getText());
			} catch (NumberFormatException e) {
				extent = 1.0;
			}
			value =rmd.getValue();
			if ( value + extent <= max) {
				rmd.setRangeProperties(value+extent,extent,min,max,false);
			} else {
				rmd.setRangeProperties(min,extent,min,max,false);
			}
		}
	}
}
