package fr.inria.oadymppac.panel;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;

/**
 * Use this modal dialog to let the user choose one string from a long
 * list.  See the main method for an example of using ListDialog.  The
 * basics:
 * <pre>
    String[] choices = {"A", "long", "array", "of", "strings"};
    ListDialog.initialize(componentInControllingFrame, choices,
                          "Dialog Title",
                          "A description of the list:");
    String selectedName = ListDialog.showDialog(locatorComponent,
                                                initialSelection);
 * </pre>
 */
public class JListDialog extends JDialog {
    static JListDialog dialog;
    static int result;
    private static JList list;
    private static JLabel label;

    /**
     * Set up the dialog.  The first argument can be null,
     * but it really should be a component in the dialog's
     * controlling frame.
     */
    public static void initialize(Component comp,
                                  Object[] possibleValues,
                                  String title,
                                  String labelText) {
        Frame frame = JOptionPane.getFrameForComponent(comp);
        dialog = new JListDialog(frame, possibleValues,
                                title, labelText);
    }

    /**
     * Show the initialized dialog.  The first argument should
     * be null if you want the dialog to come up in the center
     * of the screen.  Otherwise, the argument should be the
     * component on top of which the dialog should appear.
     */
    
    
    public static int showDialog(Component comp,  Object[] content, Object[] initialSelection) {
        if (dialog != null) {
        	setContent(content);
            setSelection(initialSelection);
            dialog.setLocationRelativeTo(comp);
            dialog.setVisible(true);
        } else {
            initialize(comp,content, "Choose", "Select items");
            setSelection(initialSelection);
            dialog.setLocationRelativeTo(comp);
            dialog.setVisible(true);
        }
        return result;
    }

    

    /**
     * 
     * Protected constructor for a Singleton
     * 
     * @param frame the parent frame this component is associated
     * @param data The possible choices
     * @param title The title of the selector window
     * @param labelText The label displayed in the selector
     */
    protected JListDialog(Frame frame, Object[] data, String title,
                       String labelText) {
        super(frame, title, true);

        //buttons
        final JButton cancelButton = new JButton("Cancel");
        final JButton okButton = new JButton("OK");
        cancelButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                dialog.setVisible(false);
                result = JOptionPane.CANCEL_OPTION;
            }
        });
        okButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                dialog.setVisible(false);
                result = JOptionPane.OK_OPTION;
            }
        });
        getRootPane().setDefaultButton(okButton);

        //main part of the dialog
        list = new JList(data);
        list.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
        list.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                if (e.getClickCount() == 2) {
                    okButton.doClick();
                }
            }
        });
        JScrollPane listScroller = new JScrollPane(list);
        listScroller.setPreferredSize(new Dimension(250, 80));
   
        listScroller.setMinimumSize(new Dimension(250, 80));
        listScroller.setAlignmentX(LEFT_ALIGNMENT);

        //Create a container so that we can add a title around
        //the scroll pane.  Can't add a title directly to the 
        //scroll pane because its background would be white.
        //Lay out the label and scroll pane from top to button.
        JPanel listPane = new JPanel();
        listPane.setLayout(new BoxLayout(listPane, BoxLayout.Y_AXIS));
        label = new JLabel(labelText);
        label.setLabelFor(list);
        listPane.add(label);
        listPane.add(Box.createRigidArea(new Dimension(0,5)));
        listPane.add(listScroller);
        listPane.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));

        //Lay out the buttons from left to right.
        JPanel buttonPane = new JPanel();
        buttonPane.setLayout(new BoxLayout(buttonPane, BoxLayout.X_AXIS));
        buttonPane.setBorder(BorderFactory.createEmptyBorder(0, 10, 10, 10));
        buttonPane.add(Box.createHorizontalGlue());
        buttonPane.add(cancelButton);
        buttonPane.add(Box.createRigidArea(new Dimension(10, 0)));
        buttonPane.add(okButton);

        //Put everything together, using the content pane's BorderLayout.
        Container contentPane = getContentPane();
        contentPane.add(listPane, BorderLayout.CENTER);
        contentPane.add(buttonPane, BorderLayout.SOUTH);

        pack();
    }

    
	/**
	 * 
	 * Set the possible choices
	 * 
	 * @param content The content to set.
	 */
	public static synchronized void setContent(Object[] content) {
		if (content != null && list != null)
			list.setListData(content);
	}
	
	/**
	 * 
	 * Return the sdelected items
	 * 
	 * @param content The content to set.
	 */
	public static synchronized Object[] getSelection() {
		return (list == null ? null : list.getSelectedValues());
	}
    
	/**
	 * 
	 * Set as selected some items
	 * 
	 * @param selected
	 */
	public static synchronized void setSelection(Object[] selected) {
		if (list != null) {
			list.clearSelection();
			if (selected !=null && selected.length >0)  {
				for(int i=0; i<selected.length; i++)  
					list.setSelectedValue(selected[i],true);
			} 
		}
	}
    
	/**
	 * 
	 * Defines the label of the selector 
	 * 
	 * @param label The label to set.
	 */
	public static synchronized void setLabel(String l) {
		label.setText(l);
	}
}
