/*
 * INRIA-OADYMPPAC
 * 
 * Ownership: Copyright (c) 1990-2004 COSYTEC S.A.
 * Author	: 	raf
 *           	COSYTEC
 *		    	4 rue Jean Rostand
 * 				91893 Orsay Cedex, France
 *				Tel. +33 1 60 19 37 38
 *				Fax. +33 1 60 19 36 20
 *
 * Purpose	: COMPLETE HERE
 * 
 * Created on 10 mars 2004
 *
 * 
 */
package fr.inria.oadymppac.parsing.event;

import org.xml.sax.Attributes;

import fr.inria.oadymppac.parsing.ContentHandler;
import fr.inria.oadymppac.parsing.ContentHandlerImpl;


/**
 * @author raf
 *
 * @
 */
public abstract class EventImpl implements Event {
   
	protected Attributes allAttributes; // special JDFK
    
    protected String buffer; 
    
    protected ContentHandler source;
    
    
    protected void buildInfo() {};

    /* (non-Javadoc)
     * @see fr.inria.oadymppac.parsing.event.Event#getValue(java.lang.String)
     */
    public String getAnyValue(String name) {
            if (allAttributes != null) {
                return allAttributes.getValue(name);
            } else {
                return null;
            }
        }
    /* (non-Javadoc)
     * @see fr.inria.oadymppac.parsing.event.Event#init(fr.inria.oadymppac.ContentHandler, org.xml.sax.Attributes)
     */
    public void init(ContentHandler handler, Attributes atts) {
    	setSource(handler);
    	this.allAttributes=atts;
    	ContentHandlerImpl.initiated++;
    }
    /* (non-Javadoc)
     * @see fr.inria.oadymppac.parsing.event.Event#init(fr.inria.oadymppac.ContentHandler, char[], int, int)
     */
    public void init(ContentHandler handler, char[] ch, int start, int length) {
    		setSource(handler);
    		ContentHandlerImpl.initiated++;
    		
    	};

        /* (non-Javadoc)
         * @see fr.inria.oadymppac.parsing.event.Event#reset()
         */
    public void terminate() {
        if (logger.isDebugEnabled()) logger.info("Terminating construction of "+this+" before firing");
        }
	
    	
    	/* (non-Javadoc)
     * @see fr.inria.oadymppac.parsing.event.Event#reset()
     */
    public void reset() {
    	setSource(null);
    	setAllAttributes(null);
    	logger.info("Event reset!");
    	ContentHandlerImpl.reset++;
    }
    
    
    protected int extractInt(String field, Attributes atts, int none) {
        if(logger.isDebugEnabled()) logger.info("Extracting "+field+" as integer");
        buffer = atts.getValue(field);
        if (buffer != null) try {
            return Integer.parseInt(buffer);
        } catch (NumberFormatException e) {
            if(logger.isDebugEnabled()) logger.warn("Invalid information provided for "+field,e);
            return none;
        }
        else {
            if(logger.isDebugEnabled()) logger.warn("No information provided for "+field);
            return none;
        }
    }
    
    protected long extractLong(String field, Attributes atts, long none) {
        if(logger.isDebugEnabled()) logger.info("Extracting "+field+" as long");
        buffer = atts.getValue(field);
        if (buffer != null) try {
            return Long.parseLong(buffer);
        } catch (NumberFormatException e) {
            if(logger.isDebugEnabled()) logger.warn("Invalid information provided for "+field,e);
            return none;
        }
        else {
            if(logger.isDebugEnabled()) logger.warn("No information provided for "+field);
            return none;
        }
    }
    
    protected int extractShort(String field, Attributes atts, short none) {
        if(logger.isDebugEnabled()) logger.info("Extracting "+field+" as short");
        buffer = atts.getValue(field);
        if (buffer != null) try {
            return Short.parseShort(buffer);
        } catch (NumberFormatException e) {
            if(logger.isDebugEnabled()) logger.warn("Invalid information provided for "+field,e);
            return none;
        }
        else {
            if(logger.isDebugEnabled()) logger.warn("No information provided for "+field);
            return none;
        }
    }
    
    protected String extractString(String field, Attributes atts, String none) {
        if(logger.isDebugEnabled()) logger.info("Extracting "+field+" as String");
        buffer = atts.getValue(field);
        if (buffer != null) {
            return buffer.intern();
        }
        else {
            if(logger.isDebugEnabled()) logger.warn("No information provided for "+field);
            return none;
        }
    }
    
    
    
	/**
	 * @return Returns the source.
	 */
	public ContentHandler getSource() {
		return source;
	}
	/**
	 * @param source The source to set.
	 */
	public void setSource(ContentHandler source) {
		this.source = source;
	}
	
    /**
     * @return Returns the allAttributes.
     */
    public Attributes getAllAttributes() {
        return allAttributes;
    }
    /**
     * @param allAttributes The allAttributes to set.
     */
    public void setAllAttributes(Attributes allAttributes) {
        this.allAttributes = allAttributes;
    }
    
    
    
}