/*
 * INRIA-OADYMPPAC
 * 
 * Ownership: Copyright (c) 1990-2004 COSYTEC S.A.
 * Author	: 	raf
 *           	COSYTEC
 *		    	4 rue Jean Rostand
 * 				91893 Orsay Cedex, France
 *				Tel. +33 1 60 19 37 38
 *				Fax. +33 1 60 19 36 20
 *
 * Purpose	: COMPLETE HERE
 * 
 * Created on 11 mars 2004
 *
 * 
 */
package fr.inria.oadymppac.parsing.gentra4cp;

import java.util.StringTokenizer;

import org.xml.sax.Attributes;

import fr.inria.oadymppac.parsing.ContentHandler;
import fr.inria.oadymppac.parsing.event.Event;
import fr.inria.oadymppac.parsing.event.Info;
import fr.inria.oadymppac.parsing.gentra4cp.event.AnnotationEvent;
import fr.inria.oadymppac.parsing.info.AnnotationInfo;


/**
 * @author raf
 *
 * @
 */
public class BasicAnnotationEvent extends BasicEvent implements AnnotationEvent,  Event
			 {
    
    private String buffer;
 
    protected AnnotationInfo info;
    private StringTokenizer tokenizer;
 
  
    /**
     * 
     */
    public BasicAnnotationEvent() {
        super();
        buildInfo();
    }

    /* (non-Javadoc)
     * @see fr.inria.oadymppac.parsing.event.Event#reset()
     */    
    protected String[] extractReferences(Attributes atts) {
        Event.logger.info("Extracting references");
        buffer = atts.getValue(REFERENCES);
        int i;
        String[] result;
        if (buffer != null) {
            tokenizer=new StringTokenizer(buffer,REFERENCES_SEPARATOR);
            i=tokenizer.countTokens();
            if (i >0 ) {
                result= new String[i];
                i=0;
            } else {
                return REFERENCES_NONE;
            }
            while(tokenizer.hasMoreTokens()) {
                result[i++]=tokenizer.nextToken();
             }
            return result;
        } else {
            return REFERENCES_NONE;
        }
    }
    
    
    /* (non-Javadoc)
     * @see fr.inria.oadymppac.parsing.event.Event#reset()
     */    
    protected short extractType(Attributes atts) {
        Event.logger.info("Extracting variable type");
        buffer = atts.getValue(EVENT_TYPE);
        if (buffer != null) {
            buffer = buffer.intern();
            if (buffer.equals(COMMAND)) {
                Event.logger.info("Command annotation detected !");
                return TYPE_COMMAND;
            } else if (buffer.equals(OBJECT)) {
                Event.logger.info("Object annotation detected !");
                return TYPE_OBJECT;
             } else {
                Event.logger.warn("Impossible to detect variable type!");
                return TYPE_NONE;
            }
        } else {
            Event.logger.warn("No variable type provided!");
            return TYPE_NONE;
        }
    }

    /* (non-Javadoc)
     * @see fr.inria.oadymppac.parsing.gentra4cp.BasicEvent#getChrono()
     */
    public int getChrono() {
        return info.getChrono();
    }
 

    /* (non-Javadoc)
     * @see fr.inria.oadymppac.parsing.gentra4cp.BasicEvent#getContext()
     */
    public String getContext() {
        return info.getContext();
    }
    /* (non-Javadoc)
     * @see fr.inria.oadymppac.parsing.gentra4cp.BasicEvent#getDepth()
     */
    public int getDepth() {
        return info.getDepth();
    }
    /* (non-Javadoc)
     * @see fr.inria.oadymppac.parsing.gentra4cp.BasicEvent#getFile()
     */
    public String getFile() {
        return info.getFile();
    }
    /**
     * @return
     */
    public String getIdentifier() {
        return info.getIdentifier();
    }
    /**
     * @return Returns the info.
     */
    public Info getInfo() {
        return (Info)info;
    }
    /* (non-Javadoc)
     * @see fr.inria.oadymppac.parsing.gentra4cp.BasicEvent#getLine()
     */
    public int getLine() {
        return info.getLine();
    }
    /**
     * @return
     */
    public String getName() {
        return info.getName();
    }
    /**
     * @return
     */
    public String[] getReferences() {
        return info.getReferences();
    }
    /* (non-Javadoc)
     * @see fr.inria.oadymppac.parsing.gentra4cp.BasicEvent#getTime()
     */
    public long getTime() {
        return info.getTime();
    }
    /**
     * @return
     */
    public short getType() {
        return info.getType();
    }
    
  
 
    
    /* (non-Javadoc)
     * @see fr.inria.oadymppac.parsing.event.Event#init(fr.inria.oadymppac.ContentHandler, org.xml.sax.Attributes)
     */
    public void init(ContentHandler handler, Attributes atts) {
    	buildInfo();
    	System.out.println("StartEvent");
        super.init(handler, atts);
        setIdentifier(extractString(IDENTIFIER,atts, IDENTIFIER_NONE));
        setName(extractString(NAME,atts, NAME_NONE));
        setType(extractType(atts));
        setReferences(extractReferences(atts));
    }
    
    /* (non-Javadoc)
     * @see fr.inria.oadymppac.parsing.event.Event#reset()
     */
    public void reset() {
    	if (info !=null) {
    		super.reset();
    		setIdentifier(IDENTIFIER_NONE);
    		setName(NAME_NONE);
    		setType(TYPE_NONE);
    		setReferences(REFERENCES_NONE);
    		info = null;
    	}
    }
    /* (non-Javadoc)
     * @see fr.inria.oadymppac.parsing.gentra4cp.BasicEvent#setChrono(int)
     */
    public void setChrono(int chrono) {
        info.setChrono(chrono);
    }
    /* (non-Javadoc)
     * @see fr.inria.oadymppac.parsing.gentra4cp.BasicEvent#setContext(java.lang.String)
     */
    public void setContext(String context) {
        info.setContext(context);
    }
    /* (non-Javadoc)
     * @see fr.inria.oadymppac.parsing.gentra4cp.BasicEvent#setDepth(int)
     */
    public void setDepth(int depth) {
        info.setDepth(depth);
    }
    /* (non-Javadoc)
     * @see fr.inria.oadymppac.parsing.gentra4cp.BasicEvent#setFile(java.lang.String)
     */
    public void setFile(String file) {
        info.setFile(file);
    }
    /**
     * @param ident
     */
    public void setIdentifier(String ident) {
        info.setIdentifier(ident);
    }
    /* (non-Javadoc)
     * @see fr.inria.oadymppac.parsing.gentra4cp.BasicEvent#setLine(int)
     */
    public void setLine(int line) {
        info.setLine(line);
    }
    /**
     * @param name
     */
    public void setName(String name) {
        info.setName(name);
    }
    /**
     * @param references
     */
    public void setReferences(String[] references) {
        info.setReferences(references);
    }
    /* (non-Javadoc)
     * @see fr.inria.oadymppac.parsing.gentra4cp.BasicEvent#setTime(long)
     */
    public void setTime(long time) {
        info.setTime(time);
    }
    /**
     * @param type
     */
    public void setType(short type) {
        info.setType(type);
    }
    
    public int getEventType() {
		return EVENT_TYPE;
	}
	/* (non-Javadoc)
	 * @see fr.inria.oadymppac.parsing.event.EventImpl#buildInfo()
	 */
	protected void buildInfo() {
		if (info==null) info = new AnnotationInfo();
	}
}