/*
 * $Id: AbstractSolver.java,v 1.5 2004/05/28 06:28:48 raf Exp $
 *
 * Copyright (c) 2001-2002 Jean-Daniel Fekete, Mohammad Ghoniem and
 * Ecole des Mines de Nantes.  All rights reserved.
 *
 * This software is proprietary information of Jean-Daniel Fekete and
 * Ecole des Mines de Nantes.  You shall use it only in accordance
 * with the terms of the license agreement you accepted when
 * downloading this software.  The license is available in the file
 * licence.txt and at the following URL:
 * http://www.emn.fr/fekete/oadymppac/licence.html
 */
package fr.inria.oadymppac.solver;

import java.util.Hashtable;
import java.util.Map;

import fr.inria.oadymppac.utils.DefaultProperties;
import fr.inria.oadymppac.utils.IndexedObjectManager;
import fr.inria.oadymppac.utils.Properties;


/**
 * This class provides the basic behavior for a <code>Solver</code>.
 * It implements all the listener management and firing logics.
 * 
 * @version $Revision: 1.5 $
 * @author Jean-Daniel Fekete
 * @author Mohammad Ghoniem
 */
public abstract class AbstractSolver  extends IndexedObjectManager implements Solver {

    protected Properties properties;

    protected int[] intValues;

    /**
     * Defines an <code>AbstractSolver</code> object.
     */
    public AbstractSolver(Properties properties) {
    	super();
    	this.properties=new DefaultProperties(properties);
    	intValues = new int[Solver.SOLVER_VALUES];
    }

  

    protected Map buildIndexer() {
		return new Hashtable(Solver.EVENT_SIZE);
	}

    
	/**
	 * @param property
	 * @return
	 */
	public Object getProperty(String property) {
		return properties.getProperty(property);
	}
	/**
	 * @param property
	 * @param value
	 */
	public void setProperty(String property, Object value) {
		properties.setProperty(property, value);
	}
	
	
	/**
	 * @return
	 */
	public Properties getProperties() {
		return properties.getProperties();
	}
	/**
	 * @param properties
	 */
	public void setProperties(Properties properties) {
		properties.setProperties(properties);
	}
	
	
	/* (non-Javadoc)
	 * @see fr.inria.oadymppac.solver.Solver#getIntValue(int)
	 */
	public int getIntValue(int index) {
		return intValues[index];
	}
	public void setIntValue(int index, int value) {
		intValues[index]= value;
	}
}