/*
 * $Id: DomainManager.java,v 1.4 2004/05/28 06:28:47 raf Exp $
 *
 * Copyright (c) 2001-2002 Jean-Daniel Fekete, Mohammad Ghoniem and
 * Ecole des Mines de Nantes.  All rights reserved.
 *
 * This software is proprietary information of Jean-Daniel Fekete and
 * Ecole des Mines de Nantes.  You shall use it only in accordance
 * with the terms of the license agreement you accepted when
 * downloading this software.  The license is available in the file
 * licence.txt and at the following URL:
 * http://www.emn.fr/fekete/oadymppac/licence.html
 */
package fr.inria.oadymppac.solver;

import java.util.Map;
import java.util.TreeMap;

import fr.inria.oadymppac.utils.Indexable;
import fr.inria.oadymppac.utils.Properties;

/**
 * A <code>VarDomainManager</code> is a utility class that manages the
 * domain of all the variables declared and used during a session.  It
 * keeps track of the domains and notifies listeners when the domains
 * change.
 *
 * @author Jean-Daniel Fekete
 * @author Mohammad Ghoniem
 * @version $Revision: 1.4 $
 */
public final class DomainManager extends OrderedManager implements Indexable  {
	private static final String PROPERTY_NAME = "DOMAIN_MANAGER";
    
    
    
    /**
     * Returns the <code>index</code>'s created variable.
     */
    public synchronized Domain get(int index) {
		return (Domain)getFromIndex(index);
    }
    
    /**
     * Returns the <code>index</code>'s created variable.
     */
    public synchronized Domain get(Object key) {
		return (Domain) getFromKey(key);
    }

   

    public static DomainManager getDomainManager(Properties owner) {
	return (DomainManager) getObjectManager(PROPERTY_NAME,DomainManager.class, owner );
    }

    
    
	
	/* (non-Javadoc)
	 * @see fr.inria.oadymppac.utils.ObjectManager#buildIndexer()
	 */
	protected Map buildIndexer() {
		return new TreeMap();
	}
	
	
	/* (non-Javadoc)
	 * @see fr.inria.oadymppac.utils.Identifiable#getIndex()
	 */
	public int getIndex() {
		return Solver.DOMAIN;
	}
	/* (non-Javadoc)
	 * @see fr.inria.oadymppac.utils.Identifiable#getKey()
	 */
	public Object getKey() {
		return PROPERTY_NAME;
	}
	/* (non-Javadoc)
	 * @see fr.inria.oadymppac.utils.Identifiable#setIndex(int)
	 */
	public void setIndex(int index) {
	}
	
	public void reset() {
		owner.setProperty(PROPERTY_NAME, null);
	}
}
