/*
 * $Id: ReduceManager.java,v 1.5 2004/05/28 06:28:48 raf Exp $
 *
 * Copyright (c) 2001-2002 Jean-Daniel Fekete, Mohammad Ghoniem and
 * Ecole des Mines de Nantes.  All rights reserved.
 *
 * This software is proprietary information of Jean-Daniel Fekete and
 * Ecole des Mines de Nantes.  You shall use it only in accordance
 * with the terms of the license agreement you accepted when
 * downloading this software.  The license is available in the file
 * licence.txt and at the following URL:
 * http://www.emn.fr/fekete/oadymppac/licence.html
 */
package fr.inria.oadymppac.solver;

import fr.inria.oadymppac.utils.Indexable;


/**
 * A <code>VariableManager</code> is a utility class that manages the
 * domain of all the variables declared and used during a session.  It
 * keeps track of the domains and notifies listeners when the domains
 * change.
 *
 * @author Jean-Daniel Fekete
 * @author Mohammad Ghoniem
 * @version $Revision: 1.5 $
 */
public final class ReduceManager extends OrderedManager implements Indexable  {
	private static final String PROPERTY_NAME = "REDUCE_MANAGER";
 
    
    
    
    /**
     * Returns the <code>index</code>'s created variable.
     */
    public synchronized Reduce get(int index) {
		return (Reduce)getFromIndex(index);
    }
    
    /**
     * Returns the <code>index</code>'s created variable.
     */
    public synchronized Reduce get(String key) {
		return (Reduce) getFromKey(key);
    }

   

    public static ReduceManager getReduceManager(Solver solver) {
	return (ReduceManager) getObjectManager(PROPERTY_NAME,ReduceManager.class, solver );
    }

   
	
	
	/* (non-Javadoc)
	 * @see fr.inria.oadymppac.utils.Identifiable#getIndex()
	 */
	public int getIndex() {
		return Solver.REDUCE;
	}
	/* (non-Javadoc)
	 * @see fr.inria.oadymppac.utils.Identifiable#getKey()
	 */
	public Object getKey() {
		return PROPERTY_NAME;
	}
	/* (non-Javadoc)
	 * @see fr.inria.oadymppac.utils.Identifiable#setIndex(int)
	 */
	public void setIndex(int index) {
	}
	
	
	/* (non-Javadoc)
	 * @see fr.inria.oadymppac.utils.IndexedObjectManager#add(fr.inria.oadymppac.utils.Indexable)
	 */
	public synchronized int add(Object element) {
		int result = super.add(element);
		return result;
	}
	
	public void reset() {
		owner.setProperty(PROPERTY_NAME, null);
	}

}
