package fr.inria.oadymppac.solver.analyser;

import infovis.Graph;
import infovis.Table;
import infovis.column.StringColumn;

import java.util.Collection;
import java.util.Iterator;

import fr.inria.oadymppac.application.ApplicationProperties;
import fr.inria.oadymppac.panel.MainFrameDecorator;
import fr.inria.oadymppac.solver.Awake;
import fr.inria.oadymppac.solver.Constraint;
import fr.inria.oadymppac.solver.ConstraintProxy;
import fr.inria.oadymppac.solver.Explanation;
import fr.inria.oadymppac.solver.Reduce;
import fr.inria.oadymppac.solver.Solver;
import fr.inria.oadymppac.solver.Update;
import fr.inria.oadymppac.solver.Variable;
import fr.inria.oadymppac.solver.VariableProxy;
import fr.inria.oadymppac.utils.DynamicIntArray;
import fr.inria.oadymppac.utils.ObjectManager;

/**
 * @author raf
 *
 * 
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
public class ConstraintConstraintGraphAnalyser extends AbstractGraphAnalyser implements SolverAnalyser {
	
	public static final String LABEL="Constraint-Constraint";
	private static final int[] TYPES = { Solver.CONSTRAINT, Solver.REDUCE};
	
	
	private boolean buildOnExplanation = false;
	private boolean buildOnAwake = false;
	private boolean buildOnReduce = false;
	

	
	DynamicIntArray constraintToNode;

	
	
	StringColumn constraint1ColumnEdge;
	StringColumn constraint2ColumnEdge;
	
	public void add(Constraint constraint) {
		try {		
			if (logger.isDebugEnabled()) logger.info("Adding a new constraint vertex corresponding to "+constraint.getIdentifier()+" at chrono="+constraint.getChrono());
			if (constraintToNode== null) constraintToNode = new DynamicIntArray();
			if (graph == null) init_graph();
			int node= graph.addVertex();
			constraintToNode.set(constraint.getIndex(),node);	
			int chrono = constraint.getChrono();
			int depth = constraint.getDepth();
			chronoColumnNode.setExtend(node,chrono);
			invChronoColumnNode.setExtend(node,-chrono);
			depthColumnNode.setExtend(node,depth);
			invDepthColumnNode.setExtend(node,-depth);
			identifierColumnNode.setExtend(node,constraint.getIdentifier());
			labelColumnNode.setExtend(node,constraint.getLabel());
			nameColumnNode.setExtend(node,constraint.getName());
			typeColumnNode.setExtend(node,constraint.getEventType());
			numTypeColumnNode.setExtend(node,Solver.CONSTRAINT);

			internalColumnNode.setExtend(node,constraint.getInternal());
			externalColumnNode.setExtend(node,constraint.getExternal());
			activationColumnNode.setValueAt(chrono, node, "0");
			invActivationColumnNode.setExtend(node, -0);
			metricColumnNode.setValueOrNullAt(chrono, node, "0");
			invMetricColumnNode.setExtend(node, -0);

		} catch (Exception exp) {
			solver.add(Solver.EXCEPTION, exp);
		}		
	}
	
	public void add(Reduce reduce) {
		buildWeightCalculator();
		Constraint ctr1 =  reduce.getConstraint();
		if (ctr1 != null && !(ctr1 instanceof ConstraintProxy) ) {
			buildOnReduce = true; // Au moins un reduce valide ?
			Collection explanations = reduce.getExplanations();
			Explanation explanation;
			Iterator it = (explanations == null ? null : explanations.iterator());
			Collection constraints;			
			while (it != null && it.hasNext() ) {
				explanation = (Explanation) it.next();
				constraints = explanation.getConstraints();
				if (constraints != null) {
					buildOnExplanation = true; //au moins un explanation-constraints  valide 
					buildOnReduce = false;
					buildOnAwake = false;
					Iterator ctrIt = constraints.iterator();
					Constraint ctr2;
					while(ctrIt.hasNext()) {
						ctr2 = (Constraint)it.next();
						if (ctr1 != ctr2) {
							initReduceEdge(ctr1,ctr2,reduce);
						}
					}
				}
			}
		}
	}

	
	public void add(Awake awake) {
		if (!buildOnExplanation) {
			Constraint ctr1 =  awake.getConstraint();
			if (ctr1 != null && !(ctr1 instanceof ConstraintProxy)) {
				Update update = awake.getUpdate();
				if (update != null) {
					buildOnAwake = true;  // au moins un awake valide
					buildOnExplanation = false;
					buildOnReduce = false;
				}
			}
		}

	}
	
	
	
	protected int add(int chrono, int index, Constraint constraint1, Constraint constraint2, int metric) {
		
		try {   	
			int ctr1Node = (constraintToNode== null ? DynamicIntArray.NONE : constraintToNode.get(constraint1.getIndex()));
			int ctr2Node = (constraintToNode== null ? DynamicIntArray.NONE : constraintToNode.get(constraint2.getIndex()));
			if (ctr1Node != DynamicIntArray.NONE && ctr2Node !=DynamicIntArray.NONE) {
				return addEdge(chrono,index, ctr1Node, ctr2Node,metric);
			} else  {
				if (logger.isDebugEnabled()) {
					logger.warn("Edge bounds not found");
					logger.warn("CTR1("+constraint1.getIdentifier()+",index="+constraint1.getIndex()+",node="+ctr1Node+")"+
							"- CTR2("+constraint2.getIdentifier()+",index="+constraint2.getIndex()+",node="+ctr2Node+")");
					logger.warn("CONSTRAINT1->NODE TABLE:"+constraintToNode);
				}
			}	
		} catch (Exception exp) {
			solver.add(Solver.EXCEPTION,exp);
		}
		return Graph.NIL;
	}
	
	/**
	 * @return Returns the graph.
	 */
	public Graph getGraph() {
		if (graph == null) init_graph();
		return graph;
	}
	/**
	 * @param graph The graph to set.
	 */
	public void setGraph(Graph graph) {
		this.graph = graph;
	}
	/**
	 * @return Returns the table.
	 */
	public Table getNodeTable() {
		return nodeTable;
	}
	/**
	 * @param table The table to set.
	 */
	public void setNodeTable(Table table) {
		this.nodeTable = table;
	}
	
	protected void init_graph() {
		super.init_graph();
		constraint1ColumnEdge = new StringColumn("Constraint1");
		edgeTable.addColumn(constraint1ColumnEdge);
		constraint2ColumnEdge = new StringColumn("Constraint2");
		edgeTable.addColumn(constraint2ColumnEdge);

		
	}
	/**
	 * @return Returns the lABEL.
	 */
	public String getLabel() {
		return LABEL;
	}
	
	
	protected int[] getAnalyseTypes() {
		return  TYPES;
	}
	/**
	 * @param solver
	 */
	public ConstraintConstraintGraphAnalyser(Solver solver) {
		super(solver);
		
	}
	
	/**
	 * @return Returns the buildOnAwake.
	 */
	public synchronized boolean isBuildOnAwake() {
		return buildOnAwake;
	}
	/**
	 * @param buildOnAwake The buildOnAwake to set.
	 */
	public synchronized void setBuildOnAwake(boolean buildOnAwake) {
		this.buildOnAwake = buildOnAwake;
	}
	/**
	 * @return Returns the buildOnExplanation.
	 */
	public synchronized boolean isBuildOnExplanation() {
		return buildOnExplanation;
	}
	/**
	 * @param buildOnExplanation The buildOnExplanation to set.
	 */
	public synchronized void setBuildOnExplanation(boolean buildOnExplanation) {
		this.buildOnExplanation = buildOnExplanation;
	}
	/**
	 * @return Returns the buildOnReduce.
	 */
	public synchronized boolean isBuildOnReduce() {
		return buildOnReduce;
	}
	/**
	 * @param buildOnReduce The buildOnReduce to set.
	 */
	public synchronized void setBuildOnReduce(boolean buildOnReduce) {
		this.buildOnReduce = buildOnReduce;
	}
	
	public void endAnalyse() {
		System.out.println("Finalizing Constraint-Constraint analysis(buildOnExplanation="+buildOnExplanation+", buildOnAwake="+buildOnAwake+", buildOnReduce="+buildOnReduce+")"); 
		if (buildOnExplanation) {
			System.out.println("Constraint-Constraint Analysis on Explanation terminated !!!");
		} else if (buildOnAwake) {
			analyseOnAwake();
		} else if (buildOnReduce) {
			analyseOnReduce();
		}
		super.endAnalyse();
	}
	
	protected void analyseOnAwake() {
		buildWeightCalculator();
		ObjectManager manager = solver.get(Solver.AWAKE);
		Iterator it;
		it = (manager == null ? null : manager.iterator());
		Awake awake;
		Constraint ctr1;
		while (it != null && it.hasNext()){
			awake = (Awake) it.next();
			ctr1 = awake.getConstraint();
			if (ctr1 != null && !(ctr1 instanceof ConstraintProxy)) {
				Update update = awake.getUpdate();
				if (update != null) {
					Variable var = (Variable)update.getElement();
					if (var != null && !(var instanceof VariableProxy)) {
						Collection constraints = var.getConstraints();
						Iterator itVar = (constraints == null ? null : constraints.iterator());
						Constraint ctr2;
						while (itVar != null && itVar.hasNext()) {
							ctr2 = (Constraint) itVar.next();
							if (ctr2 != ctr1) {
								add(awake.getChrono(),awake.getIndex(),ctr1, ctr2,1);
							}
						}
					}
				}
			}
		}
		System.out.println("Constraint-Constraint Analysis on Awake terminated !!!");
		System.gc();
	}
	
	protected void analyseOnReduce() {
		buildWeightCalculator();
		ObjectManager manager = solver.get(Solver.REDUCE);
		Iterator it,itVar;
		it = (manager == null ? null : manager.iterator());
		Reduce reduce;
		Constraint ctr1;
		MainFrameDecorator deco = (MainFrameDecorator) solver.getProperty(ApplicationProperties.APPLICATION_FRAME_DECORATOR);
		Collection constraints;
		Variable var;
		StringBuffer message;
		while (it != null && it.hasNext()){
			reduce = (Reduce) it.next();
			ctr1 = reduce.getConstraint();
			if (ctr1 != null && !(ctr1 instanceof ConstraintProxy)) {	
				var = reduce.getVariable();
				if (var != null && !(var instanceof VariableProxy)) {
					constraints = var.getConstraints();
					itVar = (constraints == null ? null : constraints.iterator());
					Constraint ctr2;
					message = new StringBuffer("Chrono:");
					message.append(reduce.getChrono());
					message.append("    ");
					message.append(reduce.getLabel());
					deco.setMessage(message.toString());
					while (itVar != null && itVar.hasNext()) {
						ctr2 = (Constraint) itVar.next();
						if (ctr1 != ctr2) {
							initReduceEdge(ctr1,ctr2,reduce);
						}
					}
				}
			}
		}
		System.out.println("Constraint-Constraint Analysis on Reduce terminated !!!");
		System.gc();
	}
	
	
	public void initReduceEdge(Constraint ctr1, Constraint ctr2, Reduce reduce) {
		int chrono, depth, metric,edge;
		metric = reduceWeightCalculator.getWeight(reduce);
		chrono=reduce.getChrono();
		depth = reduce.getDepth();
		edge = add(chrono, reduce.getIndex(),ctr1, ctr2,metric);
		if (edge != Graph.NIL) {
			chronoColumnEdge.setExtend(edge,chrono);
			invChronoColumnEdge.setExtend(edge,chrono);
			depthColumnEdge.setExtend(edge,depth);
			invDepthColumnEdge.setExtend(edge,-depth);
			labelColumnEdge.setExtend(edge,reduce.getLabel());
			typeColumnEdge.setExtend(edge,reduce.getEventType());
			constraint1ColumnEdge.setExtend(edge,ctr1.getLabel());
			constraint2ColumnEdge.setExtend(edge,ctr2.getLabel());
			numTypeColumnEdge.setExtend(edge,Solver.REDUCE);
		}
	}
}


