/*
 * Created on 13 avr. 2004
 *
 */
package fr.inria.oadymppac.utils;

import java.util.Collection;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;

/**
 * 
 * A property table reimplementation
 * Basically support only String as Key
 * Also use a HashTable => synchronized
 * 
 * 
 * @author raf
 *
 */
public class DefaultProperties implements fr.inria.oadymppac.utils.Properties {	
	
	/**
	 *  <code>parent</code> a reference to a parent property table
	 * 
	 */
	Properties parent;
	
	 /**
	 * <code>properties</code> The real content of the table
	 */
	Hashtable properties;
	
	/**
	 * 
	 * Set a new value to a property
	 * If the property table does not exist, then it is created
	 * If the property was not previously defined, it is inserted into the table
	 * If the oproperty was already define, the current value is replaced by the new one.
	 * 
	 * @see fr.inria.oadymppac.utils.Properties#setProperty(java.lang.String, java.lang.Object)
	 */
	public synchronized void setProperty(String property, Object value) {
		if (properties == null) properties = new Hashtable();
		properties.put(property,value);
	}
	
	
	/**
	 * @return the property defined for the key
	 * If the property cannot be found, then look for thisproperty in the parent table (if any)
	 * 
	 * @param property the property name
	 * 
	 * @see java.util.Properties#getProperty(java.lang.String)
	 */
	public synchronized Object getProperty(String property) {
		Object result=(this.properties == null ? null : (Object) properties.get(property));
		if (result != null)
			return result;
		else if (this.parent != null)
			return parent.getProperty(property);
		else
			if (logger.isDebugEnabled()) logger.warn("Property <"+property+"> not found in "+this);
			return null;
	}
	/** 
	 * 
	 * Return the property table
	 * 
	 * @see fr.inria.oadymppac.utils.Properties#getProperties()
	 */
	public synchronized Properties getProperties() {
		return this;
	}
	/**
	 * 
	 * Set the parent properties
	 * 
	 * @see fr.inria.oadymppac.utils.Properties#setProperties(fr.inria.oadymppac.utils.Properties)
	 */
	public void setProperties(Properties properties) {
		this.parent=properties;
	}
	
	/**
	 * Default Constructor
	 */
	public DefaultProperties() {
		this(null);
	}
	/**
	 * Build a new Property table using the given properties as parent table
	 * 
	 * @param properties the parent propety table
	 */
	public DefaultProperties(Properties properties) {
		super();
		setProperties(properties);
	}
	
	/**
	 * 
	 * Display the contnet of the property table and recursively from its parent
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		StringBuffer buffer = new StringBuffer("PROPERTY TABLE ");
		buffer.append(getClass().getName());
		buffer.append("@");
		buffer.append(Integer.toHexString(hashCode()));
		buffer.append(" :\n");
		if (properties == null) {
			buffer.append("\tNo specific properties\n");
		} else {
			Enumeration keys = properties.keys();
			Collection values = properties.values();
			Iterator it = (values == null ? null : values.iterator());
			buffer.append("\tSpecific properties:\n");
			int i=0;
			while (keys != null && keys.hasMoreElements() && it != null && it.hasNext() ) {
				i++;
				buffer.append("\t\t"+i+": ");
				buffer.append(keys.nextElement());
				buffer.append(" - ");
				buffer.append(it.next());
				buffer.append("\n");
			}
		}
		if (parent == null) {
			buffer.append("\tNo parent properties\n");
		} else {
			buffer.append("\tParent properties:");
			buffer.append(parent.toString());
		}
		return buffer.toString();
	}
}
