/*
 * $Id: IndexedObjectManager.java,v 1.5 2004/05/28 06:28:53 raf Exp $
 *
 * Copyright (c) 2001-2002 Jean-Daniel Fekete, Mohammad Ghoniem and
 * Ecole des Mines de Nantes.  All rights reserved.
 *
 * This software is proprietary information of Jean-Daniel Fekete and
 * Ecole des Mines de Nantes.  You shall use it only in accordance
 * with the terms of the license agreement you accepted when
 * downloading this software.  The license is available in the file
 * licence.txt and at the following URL:
 * http://www.emn.fr/fekete/oadymppac/licence.html
 */
package fr.inria.oadymppac.utils;

import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: </p>
 * @author Mohammad Ghoniem
 * @version $Revision: 1.5 $
 */


public abstract class IndexedObjectManager extends ObjectManagerImpl  {
	/**
     * Reference to the solver.
     */
    protected Properties owner;
    
    /**
     * This hashMap will index the XML identifiers of Variables.
     * and map them to an Integer (index in vectors)
     */
    protected Map indexer;

    /**
     * Also keep the variables in the order they have been defined.
     */
    protected Vector content;
    
    

   

    /**
     * Get the value of solver.
     * @return value of solver.
     */
    public Properties getOwner() {
	return owner;
    }

    /**
     * Set the value of solver.
     * @param v  Value to assign to solver.
     */
    public void setOwner(Properties owner) {
	this.owner = owner;
    }

 
    

    
    /**
     * protected constructor.
     */
    protected IndexedObjectManager() {
	    indexer = buildIndexer();
		content = new Vector();
    }

 

    /**
     * Returns the variable whose name is <code>vname</code>.
     */
    public synchronized int getIndex(Object key) {
        Integer index = ((Integer) indexer.get(key));
		return (index == null ? Integer.MIN_VALUE :index.intValue());
    }

    
    
    /**
     * Returns the <code>index</code>'s created variable.
     */
    public synchronized Object getFromIndex(int index) {
		return  (index == INVALID || index >= content.size() ? null :content.elementAt(index));
    }
    
    /**
     * Returns the <code>index</code>'s created variable.
     */
    public synchronized Object getFromKey(Object key) {
        int index = getIndex(key);
		return (index == Integer.MIN_VALUE ? null : content.elementAt(index));
    }

    public synchronized int add(Object element){
    	return add((Indexable) element);
    }
    
    public synchronized int add(Indexable element){
    	indexer.put(element.getKey(), new Integer(element.getIndex()));
    	return (content.add(element) ? element.getIndex() : INVALID);
    }
    
    /**
     * Return the number of variables in the manager.
     */
    public synchronized int getCount() {
		return content.size();
    }

    

    /**
     * Return the iterator on all the variables.
     */
    public synchronized Iterator iterator() {
		return content.iterator();
    }

 

   

   
    protected abstract Map buildIndexer();
    
    public String toString() {
    	StringBuffer buffer = new StringBuffer("INDEXED OBJECT MANAGER:"+this.getClass()+"@"+Integer.toHexString(hashCode())+"[\n");
    	Iterator it = iterator();
    	Indexable id;
    	int i = 0;
    	while (it != null && it.hasNext()) {
    		id = (Indexable) it.next();
    		buffer.append("\t");
    		buffer.append((id == null) ? i :id.getIndex());
    		buffer.append("\t");
    		buffer.append((id == null) ? "-" :id.getKey());
    		buffer.append("\t");
    		buffer.append((id == null ? null : id.getClass()+"@"+Integer.toHexString(id.hashCode())));
    		buffer.append("\n");
    		i++;
    	}
    	buffer.append("]");
    	return buffer.toString();
    }
   
}