/* 
 * ### Copyright (C) 2001-2003 Michael Fuchs ###
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *      
 * Author: Michael Fuchs
 * E-Mail: mfuchs@unico-consulting.com
 *
 * RCS Information:
 * ---------------
 * Id.........: $Id: BooleanOption.java,v 1.1.1.1 2003/10/21 19:19:15 cvs Exp $
 * Author.....: $Author: cvs $
 * Date.......: $Date: 2003/10/21 19:19:15 $
 * Revision...: $Revision: 1.1.1.1 $
 * State......: $State: Exp $
 */
package com.mf.option;

/**
 * <div lang="en">
 * The class <code>BooleanOption</code> implements a flag option with no
 * argument.  
 *
 * If the flag is present on the command line, the value of the option will be
 * true, otherwise it will be false.
 *
 * </div>
 *
 * <div lang="de"> Die Klasse <code>BooleanOption</code> realisiert die
 * Standardoption ohne Argument.
 *
 * Wird die Option auf der Kommandozeile gesetzt, so erhlt die Option den Wert
 * "wahr". Andernfalls den Wert "falsch". Der Standardwert kann bei Aufruf der
 * Methode <code>getFlag()</code> berschrieben werden.
 *
 * </div>
 *
 * <div id="example_BooleanOption" 
 *      title="BooleanOption">
 * <pre>
 OptionList options = new OptionList(args);

 Option option = new BooleanOption("help","h");
 options.add(option);

 if ( options.getFlag("help",false) ) {
     printUsage();
     return;
 } 

 * </pre>
 * </div>
 *
 * @author <a href="mailto:mfuchs@unico-consulting.com">Michael Fuchs</a>
 * @version 1.0
 */
public class BooleanOption extends Option {

    /**
     * Creates a new <code>BooleanOption</code> instance.
     */
    public BooleanOption() {
        super();
        setDefault("false");
    }

    /**
     * <div lang="en">
     * Creates a new <code>BooleanOption</code> instance.
     * </div>
     *
     * <div lang="de">
     * Erzeugt eine neue Instanz der Klasse <code>BooleanOption</code>.
     * </div>
     *
     * @param longName a <code>String</code> value
     * @param shortName a <code>String</code> value
     */
    public BooleanOption(String longName, String shortName) {
        super(longName, shortName);
        setDefault("false");
    }

    public void setValue(String value) {

        if (value.equals("1") 
            || value.equalsIgnoreCase("yes")
            || value.equalsIgnoreCase("true")
            || value.equalsIgnoreCase("ja")
            || value.equalsIgnoreCase("wahr")) {

            value = "true";
        } else {
            value = "false";
        } // end of else
        
        super.setValue(value);
    }
}
/*
 * $Log: BooleanOption.java,v $
 * Revision 1.1.1.1  2003/10/21 19:19:15  cvs
 * Getopts library.
 *
 * Revision 1.1  2003/08/13 18:39:47  mfuchs
 * Option
 *
 * Revision 1.1  2003/08/12 19:28:10  mfuchs
 * log4j properties files for differnet logging levels.
 *
 */
