/* 
 * ### Copyright (C) 2001-2003 Michael Fuchs ###
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA.
 *      
 * Author: Michael Fuchs
 * E-Mail: mfuchs@unico-consulting.com
 *
 * RCS Information:
 * ---------------
 * Id.........: $Id: StringOption.java, v 1.1.1.1 2003/10/21 19:19:15 cvs Exp $
 * Author.....: $Author: cvs $
 * Date.......: $Date: 2003/10/21 19:19:15 $
 * Revision...: $Revision: 1.1.1.1 $
 * State......: $State: Exp $
 */
package com.mf.option;

import java.util.ResourceBundle;

/**
 * Describe class <code>IntegerOption</code> here.
 *
 * @author <a href="mailto:mfuchs@unico-consulting.com">Michael Fuchs</a>
 * @version 1.0
 */
public class IntegerOption extends Option {

    /**
     * Creates a new <code>IntegerOption</code> instance.
     *
     */
    public IntegerOption() {
        super();
        hasArgument(true);
    }

    /**
     * Creates a new <code>IntegerOption</code> instance.
     *
     * @param longName a <code>String</code> value
     * @param shortName a <code>String</code> value
     */
    public IntegerOption(String longName, String shortName) {
        super(longName, shortName);
        hasArgument(true);
    }

    /**
     * Describe <code>setInteger</code> method here.
     *
     * @param num an <code>int</code> value
     */
    public void setInteger(int num) {
        setValue(String.valueOf(num));
    }

    /**
     * Describe <code>getInteger</code> method here.
     *
     * @return an <code>int</code> value
     * @exception OptionException if an error occurs
     */
    public int getInteger() 
        throws OptionException {

        ResourceBundle res = getResourceBundle();
        String value = getValue();
        
        if (value != null) {
            Integer num;
            
            try {

                num = new Integer(value);
                return num.intValue();
                
            } catch (NumberFormatException oops) {

                throw new OptionException
                    (MessageServices.format(res.getString("C_ERROR_INVALID_NUMBER"), 
                                            getLongName(), 
                                            value));
            }

        } // end of if (value != null)
    
        return 0;
    }
}
/*
 * $Log: StringOption.java, v $
 * Revision 1.1.1.1  2003/10/21 19:19:15  cvs
 * Getopts library.
 *
 * Revision 1.1  2003/08/13 18:39:47  mfuchs
 * Option
 *
 * Revision 1.1  2003/08/12 19:28:10  mfuchs
 * log4j properties files for differnet logging levels.
 *
 */
