/* 
 * ### Copyright (C) 2001-2003 Michael Fuchs ###
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA.
 *      
 * Author: Michael Fuchs
 * E-Mail: mfuchs@unico-consulting.com
 *
 * RCS Information:
 * ---------------
 * Id.........: $Id: Option.java, v 1.1.1.1 2003/10/21 19:19:15 cvs Exp $
 * Author.....: $Author: cvs $
 * Date.......: $Date: 2003/10/21 19:19:15 $
 * Revision...: $Revision: 1.1.1.1 $
 * State......: $State: Exp $
 */
package com.mf.option;

import java.util.PropertyResourceBundle;
import java.util.ResourceBundle;
import java.util.Iterator;
import java.util.ArrayList;

/**
 * Describe class <code>Option</code> here.
 *
 * @author <a href ="mailto:mfuchs@unico-consulting.com">Michael Fuchs</a>
 * @version 1.0
 */
public abstract class Option implements Comparable {

    /**
     * Describe variable <code>res</code> here.
     *
     */
    private static ResourceBundle res = PropertyResourceBundle.getBundle("com.mf.option.Resources");
    
    /**
     * Describe variable <code>longName</code> here.
     *
     */
    private String longName;

    /**
     * Describe variable <code>shortName</code> here.
     *
     */
    private String shortName;

    /**
     * Describe variable <code>mediumName</code> here.
     *
     */
    private String mediumName;

    /**
     * Describe variable <code>propertyName</code> here.
     *
     */
    private String propertyName;

    /**
     * Describe variable <code>value</code> here.
     *
     */
    private ArrayList values = new ArrayList();

    /**
     * Describe variable <code>isPresent</code> here.
     *
     */
    private boolean isPresent = false;

    /**
     * Describe variable <code>isRequired</code> here.
     *
     */
    private boolean isRequired = false;

    /**
     * The variable <code>isUnique</code> defines wether the option may occure
     * multiple times on the command line or not.
     */
    private boolean isUnique = true;

    /**
     * Describe variable <code>hasArgument</code> here.
     *
     */
    private boolean hasArgument = false;

    /**
     * Describe variable <code>isLowerCase</code> here.
     *
     */
    private boolean isLowerCase = false;

    /**
     * Describe variable <code>isUnset</code> here.
     *
     */
    private boolean isUnset = true;
    
    /**
     * Creates a new <code>Option</code> instance.
     *
     */
    public Option() {
        
        this.longName = "";
        this.shortName = "";
        this.mediumName = "";
        this.propertyName = "";

        this.values = new ArrayList();
    }

    public Option(String longName) {

        if (longName == null) {
            throw new IllegalArgumentException("Variable longName is null!");
        }
        
        this.longName = longName;
        this.shortName = "";
        this.mediumName = "";
        this.propertyName = "";
        this.values = new ArrayList();
    }

    /**
     * Creates a new <code>Option</code> instance.
     *
     * @param longName a <code>String</code> value
     * @param shortName a <code>String</code> value
     */
    public Option(String longName, 
                  String shortName) {

        if (longName == null) {
            throw new IllegalArgumentException("Variable longName is null!");
        }
        
        if (shortName == null) {
            throw new IllegalArgumentException("Variable shortName is null!");
        }
        
        this.longName = longName;
        this.shortName = shortName;
        this.mediumName = "";
        this.propertyName = "";
        this.values = new ArrayList();
    }

    /**
     * Describe <code>getResourceBundle</code> method here.
     *
     * @return a <code>ResourceBundle</code> value
     */
    public static ResourceBundle getResourceBundle() {
        return res;
    }
    
    /**
     * Describe <code>getNames</code> method here.
     *
     * @return a <code>String[]</code> value
     */
    public String[] getNames() {
        
        ArrayList list = new ArrayList();
        
        if (longName != null 
            && longName.length() > 0
            && list.contains(longName) == false) {
            list.add(longName);
        }
        
        if (mediumName != null 
            && mediumName.length() > 0
            && list.contains(mediumName) == false) {
            list.add(mediumName);
        }
        
        if (shortName != null 
            && shortName.length() > 0
            && list.contains(shortName) == false) {
            list.add(shortName);
        }
        
        if (propertyName != null 
            && propertyName.length() > 0
            && list.contains(propertyName) == false) {
            list.add(propertyName);
        }

        String[] names = new String[list.size()]; 
        for (int i = 0; i<list.size(); i++) {
            names[i] = (String) list.get(i);
        }        

        return names;
    }

    /**
     * Describe <code>setLongName</code> method here.
     *
     * @param longName a <code>String</code> value
     */
    public void setLongName(String longName) {
        
        if (longName == null) {
            throw new IllegalArgumentException(" The argument longName may not be null!");
        }
 
        this.longName = longName;
    }

    /**
     * Describe <code>getLongName</code> method here.
     *
     * @return a <code>String</code> value
     */
    public String getLongName() {
        return longName;
    }
    
    /**
     * Describe <code>setShortName</code> method here.
     *
     * @param shortName a <code>String</code> value
     */
    public void setShortName(String shortName) {
 
        if (shortName == null) {
            throw new IllegalArgumentException(" The argument shortName may not be null!");
        }
 
        this.shortName = shortName;
    }
    
    /**
     * Describe <code>getShortName</code> method here.
     *
     * @return a <code>String</code> value
     */
    public String getShortName() {
        return shortName;
    }

    /**
     * Describe <code>setMediumName</code> method here.
     *
     * @param mediumName a <code>String</code> value
     */
    public void setMediumName(String mediumName) {

        if (mediumName == null) {
            throw new IllegalArgumentException(" The argument mediumName may not be null!");
        }
 
        this.mediumName = mediumName;
    }
    
    /**
     * Describe <code>getMediumName</code> method here.
     *
     * @return a <code>String</code> value
     */
    public String getMediumName() {
        return mediumName;
    }
 
    /**
     * Describe <code>setPropertyName</code> method here.
     *
     * @param propertyName a <code>String</code> value
     */
    public void setPropertyName(String propertyName) {

        if (propertyName == null) {
            throw new IllegalArgumentException(" The argument propertyName may not be null!");
        }
 
        this.propertyName = propertyName;
    }
    
    /**
     * Describe <code>getPropertyName</code> method here.
     *
     * @return a <code>String</code> value
     */
    public String getPropertyName() {
        return propertyName;
    }
 
    /**
     * Describe <code>getUniqueName</code> method here.
     *
     * @return a <code>String</code> value
     */
    public String getUniqueName() {

        if (longName != null && longName.length() > 0) {
            return "--" + longName;
        }
        
        if (mediumName != null && mediumName.length() > 0) {
            return "-" + mediumName;
        }
        
        if (shortName != null && shortName.length() > 0) {
            return "-" + shortName;
        }
        
        if (propertyName != null && propertyName.length() > 0) {
            return propertyName;
        }
        
        return "???";
    }
 
    public String getFQName() {

        String buffer = "(";

        if (propertyName != null && propertyName.length() > 0) {

            if (buffer.length() > 0 ) {
                buffer += ".";
            }
            
            buffer += propertyName;
        }

        if (longName != null && longName.length() > 0) {

            if (buffer.length() > 0 ) {
                buffer += "/";
            }
            
            buffer += longName;
        }
        
        if (mediumName != null && mediumName.length() > 0) {

            if (buffer.length() > 0 ) {
                buffer += "/";
            }
            
            buffer += mediumName;
        }
        
        if (shortName != null && shortName.length() > 0) {

            if (buffer.length() > 0 ) {
                buffer += "/";
            }
            
            buffer += shortName;
        }

        buffer += ")";
        
        return buffer;
    }

    /**
     * Describe <code>setDefault</code> method here.
     *
     * @param value a <code>String</code> value
     */
    public void setDefault(String value) {

        if (value == null) {
            throw new IllegalArgumentException(" The argument value may not be null!");
        }
 
        values.clear();
        values.add(value);
    }

    /**
     * Describe <code>setLowerCase</code> method here.
     *
     * @param lowerCase a <code>boolean</code> value
     */
    public void setLowerCase(boolean lowerCase) {
        this.isLowerCase = isLowerCase;
    }
    
    /**
     * Describe <code>isLowerCase</code> method here.
     *
     * @return a <code>boolean</code> value
     */
    public boolean isLowerCase() {
        return isLowerCase;
    }
    
    /**
     * Describe <code>isUnset</code> method here.
     *
     * @return a <code>boolean</code> value
     */
    public boolean isUnset() {
        return isUnset;
    }
    
    public void isUnset(boolean isUnset) {
        this.isUnset = isUnset;
    }
    
    /**
     * Describe <code>setValue</code> method here.
     *
     * @param value a <code>String</code> value
     */
    public void setValue(String value) {

        if (value == null) {
            throw new IllegalArgumentException("Variable value is null!");
        }
        
        values.clear();
        values.add(value);

        isUnset = false;
    }
    
    /**
     * The method <code>getValue</code> returns the value of the option.
     *
     * <ul>
     *   <li>If the option has no value, an empty String is returned.</li>
     *   <li>If the option has multiple values, the first one is returned.<li>
     * </ul>
     *
     * <div id="example_option_getValue" title="Example">
     * <p>If the command line looks like</p>
     * <pre>cmd -tag a -tag b -tag c</pre>
     * <p>the method getValue() for option tag will return 'a'.</p>
     * </div>
     *
     * @return a <code>String</code> value
     */
    public String getValue() {

        String value = "";
        
        if (values.size() > 0) {

             Object obj = values.get(0);

             if (obj instanceof String) {
                 value = (String) obj;
             }
        }
        
        return value;
    }
    
    protected void addValue(String value) {

        if (value == null) {
            throw new IllegalArgumentException("Variable value is null!");
        }
        
        values.add(value);
        isUnset = false;

    }

    public ArrayList getValues() {
        return values;
    }
    
    /**
     * Describe <code>getFlag</code> method here.
     *
     * @return a <code>boolean</code> value
     */
    public boolean getFlag() {
        return new Boolean(getValue()).booleanValue();
    }

    public void setFlag(boolean flag) {
        setValue(String.valueOf(flag));
    }
    

    /**
     * Describe <code>isRequired</code> method here.
     *
     * @param value a <code>boolean</code> value
     */
    public void isRequired(boolean value) {
        isRequired = value;
    }

    /**
     * Describe <code>isRequired</code> method here.
     *
     * @return a <code>boolean</code> value
     */
    public boolean isRequired() {
        return isRequired;
    }

    /**
     * Describe <code>isUnique</code> method here.
     *
     * @param value a <code>boolean</code> value
     */
    public void isUnique(boolean value) {
        isUnique = value;
    }

    /**
     * The method <code>isUnique</code> returns false, if the option may occure
     * multiple times on the comand line.
     *
     * @return a <code>boolean</code> value
     */
    public boolean isUnique() {
        return isUnique;
    }

    /**
     * Describe <code>isValid</code> method here.
     *
     * @return a <code>boolean</code> value
     */
    public boolean isValid() {

        if (longName == null) {
            return false;
        }
        
        if (mediumName == null) {
            return false;
        }
        
        if (shortName == null) {
            return false;
        }
        
        if (propertyName == null) {
            return false;
        }

        if (longName.length() == 0 
            && mediumName.length() == 0
            && shortName.length() == 0
            && propertyName.length() == 0) {
            return false;
        } // end of if (longName.length() == 0)
        
        return true;
    }

    /**
     * Describe <code>equals</code> method here.
     *
     * @param obj an <code>Object</code> value
     * @return a <code>boolean</code> value
     */
    public boolean equals(Object obj) {

        if (obj == null) {
            return false;
        }
        
        if (obj instanceof Option) {
            
            Option option = (Option) obj;

            if (isValid() == false) {
                return false;
            }
        
            if (option.isValid() == false) {
                return false;
            }
        
            if (longName.equals(option.getLongName()) 
                && mediumName.equals(option.getMediumName())
                && shortName.equals(option.getShortName())
                && propertyName.equals(option.getPropertyName())) {

                return true;
            }
        }

        return false;
    }

    /**
     * Describe <code>hashCode</code> method here.
     *
     * @return an <code>int</code> value
     */
    public int hashCode() {

        int result = 17;

        result = 37 * result + longName.hashCode();
        result = 37 * result + mediumName.hashCode();
        result = 37 * result + shortName.hashCode();
        result = 37 * result + propertyName.hashCode();

        return result;
    }

    /**
     * Describe <code>toString</code> method here.
     *
     * @return a <code>String</code> value
     */
    public String toString() {

        StringBuffer buffer = new StringBuffer();
        String value;

        if (propertyName != null && propertyName.length() > 0) {
            buffer.append(propertyName + " ");
        }
        
        if (longName != null && longName.length() > 0) {
            buffer.append(longName + " ");
        }

        if (mediumName != null && mediumName.length() > 0) {
            buffer.append(mediumName + " ");
        }

        if (shortName != null && shortName.length() > 0) {
            buffer.append(shortName + " ");
        }

        buffer.append("[");
             
        if (hasArgument) {
            buffer.append('A');
        } else {
            buffer.append('a');
        } // end of else
        
        if (isPresent) {
            buffer.append('P');
        } else {
            buffer.append('p');
        } // end of else
        
        if (isRequired) {
            buffer.append('R');
        } else {
            buffer.append('r');
        } // end of else
        
        buffer.append(']');
        for (int j = buffer.length(); j < 32; j++) {
            buffer.append('.');
        }
        
        buffer.append(": ");

        int counter = 0;
        for (Iterator i = values.iterator(); i.hasNext();) {

            value = (String) i.next();
            
            if (counter > 0) {
                buffer.append(", ");
            } // end of if (counter > 0)
            
            buffer.append(value);
            counter++;

        } // end of for (Iterator i = values.iterator(); i.hasNext();)
        
        return buffer.toString();
    }

    /**
     * Describe <code>isPresent</code> method here.
     *
     * @param value a <code>boolean</code> value
     */
    protected void isPresent(boolean value) {
        isPresent = value;
    }

    /**
     * Describe <code>hasArgument</code> method here.
     *
     * @param value a <code>boolean</code> value
     */
    protected void hasArgument(boolean value) {
        hasArgument = value;
    }
    
    /**
     * Describe <code>hasArgument</code> method here.
     *
     * @return a <code>boolean</code> value
     */
    protected boolean hasArgument() {
        return hasArgument;
    }
    
    /**
     * Describe <code>checkArgument</code> method here.
     *
     * @param arg a <code>String</code> value
     * @return a <code>boolean</code> value
     */
    protected boolean checkArgument(String arg) {
        return true;
    }

    
    // Implementation of java.lang.Comparable

    /**
     * Describe <code>compareTo</code> method here.
     *
     * @param object an <code>Object</code> value
     * @return an <code>int</code> value
     */
    public int compareTo(Object object) {
        
        if (object == null) {
            return 1;
        }
        
        if (object instanceof Option) {

            Option option = (Option) object;
            String name = getFQName();
            return name.compareTo(option.getFQName());

        } else {
            throw new ClassCastException(object.getClass().getName());
        } // end of else
    }
    
}
/*
 * $Log: Option.java, v $
 * Revision 1.1.1.1  2003/10/21 19:19:15  cvs
 * Getopts library.
 *
 * Revision 1.1  2003/08/13 18:39:47  mfuchs
 * Option
 *
 * Revision 1.1  2003/08/12 19:28:10  mfuchs
 * log4j properties files for differnet logging levels.
 *
 */
