/* 
 * ### Copyright (C) 2001-2003 Michael Fuchs ###
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA.
 *      
 * Author: Michael Fuchs
 * E-Mail: mfuchs@unico-consulting.com
 *
 * RCS Information:
 * ---------------
 * Id.........: $Id: OptionList.java, v 1.1.1.1 2003/10/21 19:19:15 cvs Exp $
 * Author.....: $Author: cvs $
 * Date.......: $Date: 2003/10/21 19:19:15 $
 * Revision...: $Revision: 1.1.1.1 $
 * State......: $State: Exp $
 */
package com.mf.option;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Properties;
import java.util.ResourceBundle;

/**
 * Describe class <code>OptionList</code> here.
 *
 * @author <a href="mailto:mfuchs@unico-consulting.com">Michael Fuchs</a>
 * @version 1.0
 */
public class OptionList {

    private static ResourceBundle res = Option.getResourceBundle();

    private ArrayList args;

    private ArrayList optList;

    private HashMap optMap;

    private HashMap deprecatedPropertiesMap;

    private String error = "";

    public OptionList() {

        String[] args = new String[0];
        init(args);
    }

    /**
     * Creates a new <code>OptionList</code> instance.
     *
     * @param cmdline a <code>String[][]</code> value
     */
    public OptionList(String [][] cmdline) {
        
        ArrayList list = new ArrayList();
            
        for (int i = 0; i < cmdline.length; i++) {
            for (int j = 0; j < cmdline[i].length; j++) {
                list.add(cmdline[i][j]);
            }
        }

        String[] args = new String[list.size()];
        
        int index = 0;
        for (Iterator i = list.iterator(); i.hasNext();) {
            args[index++] = (String) i.next();
        } // end of for (Iterator i = list.iterator(); i.hasNext();)
        
        init(args);
    }
    
    /**
     * Creates a new <code>OptionList</code> instance.
     *
     * @param cmdline a <code>String[]</code> value
     */
    public OptionList(String [] cmdline) {

        if (cmdline == null)
            throw new IllegalArgumentException("Variable args is null!");

        init(cmdline);
    }
    
    /**
     * Describe <code>getError</code> method here.
     *
     * @return a <code>String</code> value
     */
    public String getError() {

        if (error == null)
            error = "";
        
        return error;
    }
    
    /**
     * Describe <code>find</code> method here.
     *
     * @param longName a <code>String</code> value
     * @param shortName a <code>String</code> value
     * @return an <code>Option</code> value
     */
    public Option findOption(Option option) {

        Object obj;

        int index = optList.indexOf(option);
        
        if (index != -1) {
        
            obj = optList.get(index);
            if (obj != null && obj instanceof Option) {
                return (Option) obj;
            }
        }

        return null;
    }
    
    /**
     * The method <code>findOption</code> tries to find the given option by
     * name. If the option can not be found, null is returned. 
     *
     * @param name a <code>String</code> value
     * @return The option or null if no option can be found.
     */
    public Option findOption(String name) {

        if (name == null) {
            throw new IllegalArgumentException(" The argument name may not be null!");
        }
 
        return (Option) optMap.get(name);
    }

    public Option findExistingOption(Option option) {

        if (option == null) {
            throw new IllegalArgumentException(" The argument option may not be null!");
        }
 
        Option existing;

        existing = (Option) optMap.get(option.getLongName());
        if (existing != null) {
            return existing;
        }

        existing = (Option) optMap.get(option.getMediumName());
        if (existing != null) {
            return existing;
        }

        existing = (Option) optMap.get(option.getShortName());
        if (existing != null) {
            return existing;
        }

        existing = (Option) optMap.get(option.getPropertyName());
        if (existing != null) {
            return existing;
        }

        throw new OptionException("Can't find existing option!");
    }

    /**
     * The method <code>getOption</code> tries to find the given option by
     * name. If the option can not be found, an OptionException is thrown.
     *
     * @param name a <code>String</code> value
     * @return The option.
     */
    public Option getOption(String name) 
        throws OptionException {

        if (name == null) {
            throw new IllegalArgumentException("The argument name may not be null!");
        }

        Option option = (Option) optMap.get(name);

        if (option == null) {

            String msg = MessageServices.format(res.getString("C_ERROR_OPTION_UNKNOWN"), 
                                                name);
            throw new OptionException(msg);
        }
        
        return option;
    }

    public BooleanOption getBooleanOption(String name) {

        if (name == null) {
            throw new IllegalArgumentException("The argument name may not be null!");
        }
 
        Option option = getOption(name);

        if (option instanceof BooleanOption) {
            return (BooleanOption) option;
        } else {
             
            String msg = MessageServices.format(res.getString("C_ERROR_OPTION_INVALID_TYPE"), 
                                                name,
                                                "boolean",
                                                option.getClass().getName());
            throw new OptionException(msg);
        } // end of else
    }
        
    /**
     * Describe <code>add</code> method here.
     *
     * @param option an <code>Option</code> value
     * @exception OptionException if an error occurs
     */
    public void add(Option option) 
        throws OptionException {

        if (option == null) {
            throw new IllegalArgumentException(" The argument option may not be null!");
        }
 
        Option opt;

        if (option.isValid() == false) {
            throw new OptionException(res.getString("C_ERROR_INVALID_OPTION") + option.getUniqueName());
        }
        
        if (exists(option)) {

            Option existing = findExistingOption(option);

            throw new OptionException
                (MessageServices.format
                 (res.getString("C_ERROR_OPTION_ALREADY_IN_USE"), option.getFQName(), existing.getFQName()));
            
        }
        
        optList.add(option);

        String[] names = option.getNames();

        for (int i = 0; i < names.length; i++) {
            optMap.put(names[i],option);
        }
        
    }
    
    /**
     * Describe <code>getFlag</code> method here.
     *
     * @param name a <code>String</code> value
     * @param def a <code>boolean</code> value
     * @return a <code>boolean</code> value
     */
    public boolean getFlag(String name, boolean def) {

        if (name == null)
            throw new IllegalArgumentException("Variable name is null!");
        
        Option option = (Option) optMap.get(name);
        if (option == null)
            return def;
        else
            return option.getFlag();
    }
    
    /**
     * Describe <code>getString</code> method here.
     *
     * @param name a <code>String</code> value
     * @return a <code>String</code> value
     */
    public String getString(String name) {

        if (name == null)
            throw new IllegalArgumentException("Variable name is null!");
        
        Option option = (Option) optMap.get(name);
        if (option == null) {
            // throw new OptionException("Can't find option " + name + "!");
            return "";
        }

        return option.getValue();
    }
    
    /**
     * Describe <code>getString</code> method here.
     *
     * @param name a <code>String</code> value
     * @param deflt a <code>String</code> value
     * @return a <code>String</code> value
     */
    public String getString(String name, String deflt) {

        if (name == null)
            throw new IllegalArgumentException("Variable name is null!");
        
        Option option = (Option) optMap.get(name);

        if (option == null || option.isUnset())
            return deflt;
        
        String value = option.getValue();

        if (value == null) {
            return deflt;
        } else {
            return value;
        } // end of else
        
    }
    
    /**
     * Describe <code>getInteger</code> method here.
     *
     * @param name a <code>String</code> value
     * @return an <code>int</code> value
     */
    public int getInteger(String name) {

        if (name == null)
            throw new IllegalArgumentException("Variable name is null!");
        
        Option option = (Option) optMap.get(name);

        if (option == null)
            throw new OptionException("Can't find option " + name + "!");

        if (option instanceof IntegerOption) {
            return ((IntegerOption) option).getInteger();
        } else {
            return Integer.parseInt(option.getValue());
        } // end of else
    }
    
    /**
     * Describe <code>addDeprecatedPropertyAlias</code> method here.
     *
     * @param option an <code>Option</code> value
     * @param alias a <code>String</code> value
     */
    public void addDeprecatedPropertyAlias(Option option, String alias)
        throws OptionException {

        if (option == null) {
            throw new IllegalArgumentException("The argument option may not be null!");
        }
 
        if (alias == null || alias.length() == 0 || alias.equals("!")) {
            throw new IllegalArgumentException("The argument alias is invalid: " + alias);
        }
 
        if (deprecatedPropertiesMap == null) {
            throw new IllegalStateException(" The field deprecatedPropertiesMap may not be null!");
        }

        String name = alias;
        if (name.startsWith("!")) {
            name = name.substring(1);
        } // end of if (name.startsWith("!"))
        
        Option opt = (Option) optMap.get(name);

        if (opt != null && opt.equals(option) == false) {
            throw new OptionException
                (MessageServices.format
                 (res.getString("C_ERROR_OPTION_ALREADY_IN_USE"), name));
            
        }
        
        deprecatedPropertiesMap.put(alias,option);
    }
    
    /**
     * Describe <code>setProperties</code> method here.
     *
     * @param fname a <code>String</code> value
     * @exception FileNotFoundException if an error occurs
     * @exception IOException if an error occurs
     */
    public boolean setProperties(String fname) 
        throws FileNotFoundException, 
               IOException {

        if (fname == null) {
            throw new IllegalArgumentException(" The argument fname may not be null!");
        }
 
        Properties props = new Properties();
        props.load(new FileInputStream(fname));

        return setProperties(props);
    }

    /**
     * Describe <code>setProperties</code> method here.
     *
     * @param fname a <code>String</code> value
     * @exception FileNotFoundException if an error occurs
     * @exception IOException if an error occurs
     */
    public boolean setProperties(Properties props) 
        throws OptionException {
        
        if (props == null) {
            throw new IllegalArgumentException(" The argument props may not be null!");
        }
 
        Object obj;
        Option option;
        String name;
        String value;
        
        String buffer = "";
        
        for (Iterator i = props.keySet().iterator(); i.hasNext();) {

            obj = i.next();
            
            if (obj instanceof String) {

                name = (String) obj;
                value = props.getProperty(name);

                if (value == null) {
                    continue;
                }
                
                option = (Option) optMap.get(name);
                
                if (option == null) {

                    option = (Option) deprecatedPropertiesMap.get(name);
                    
                    if (option != null) {
                        buffer += MessageServices.format(res.getString("C_WARN_PROPERTY_DEPRECATED"), 
                                                         name,
                                                         option.getPropertyName()) + "\n";
                    } else {
                         
                        option = (Option) deprecatedPropertiesMap.get("!" + name);
                        
                        if (option != null && (option instanceof BooleanOption)) {

                            value = value.trim();
                            
                            boolean flag = false;
            
                            if (value.equals("1") 
                                || value.equalsIgnoreCase("yes")
                                || value.equalsIgnoreCase("true")
                                || value.equalsIgnoreCase("ja")
                                || value.equalsIgnoreCase("wahr")) {

                                flag = true;
                            }

                            option.setValue(String.valueOf(!flag));
                            continue;
                        }
                    }
                }
                
                if (option == null) {

                    buffer += MessageServices.format(res.getString("C_ERROR_PROPERTY_UNKNOWN"), 
                                                     name) + "\n";
                    continue;
                }

                if (option != null && option.isUnset()) {
                    option.setValue(value);
                }
            }
        }

        if (buffer != null && buffer.length() > 0) {
            error = buffer;
            return false;
        } else {
            return true;
        }
    }

    public boolean validate() {
        return validate(false);
    }


    /**
     * Describe <code>validate</code> method here.
     *
     * @return a <code>boolean</code> value
     * @exception OptionException if an error occurs
     */
    public boolean validate(boolean ignoreUnknownOptions) 
        throws OptionException {

        Iterator iterator = optList.iterator();
        Option option;
        String arg;
        String str;
        
        String buffer = "";
        int index = -1;
        int counter = 0;

        ArrayList argList = new ArrayList(args);

        while (iterator.hasNext()) {
            
            option = (Option) iterator.next();
            option.isUnset(true);

            index = searchArgs(argList, option);

            if (index == -1 && option.isRequired()) {
                buffer += MessageServices.format(res.getString("C_ERROR_OPTION_REQUIRED"), 
                                                 option.getUniqueName()) + "\n";
                continue;
            }
            
            counter = 0;

            while (index != -1) {
                
                if (option.hasArgument()) {
                    
                    if (index == argList.size()-1) {
                        buffer += MessageServices.format(res.getString("C_ERROR_OPTION_NEEDS_ARG"), 
                                                         option.getUniqueName()) + "\n";
                        argList.remove(index);

                        index = searchArgs(argList, option);
                        continue;
                    }
                    
                    arg = (String) argList.get(index + 1);
                    
                    if (arg.startsWith("-")) {
                        buffer += MessageServices.format(res.getString("C_ERROR_OPTION_NEEDS_ARG"), 
                                                         option.getUniqueName()) + "\n";
                        argList.remove(index);

                        index = searchArgs(argList, option);
                        continue;
                    }

                    if (option instanceof SelectOption) {
                        
                        SelectOption selopt = (SelectOption) option;
                        
                        if (selopt.checkArgument(arg) == false) {
                            buffer += MessageServices.format(res.getString("C_ERROR_OPTION_INVALID_VALUE"), 
                                                             option.getUniqueName(), 
                                                             arg, 
                                                             selopt.getListAsString()) + "\n";
                            argList.remove(index + 1);
                            argList.remove(index);

                            index = searchArgs(argList, option);
                            continue;
                        }
                        
                    }
                    
                    if (option instanceof PathOption) {
                        
                        PathOption pathopt = (PathOption) option;
                        
                        if (pathopt.checkArgument(arg) == false) {
                            buffer += MessageServices.format(res.getString("C_ERROR_OPTION_INVALID_PATH"), 
                                                             option.getUniqueName(), 
                                                             arg, 
                                                             pathopt.getInvalidPathElement()) 
                                + "\n";
                            
                            argList.remove(index + 1);
                            argList.remove(index);

                            index = searchArgs(argList, option);
                            continue;
                        }
                    }
                        
                    if (option instanceof FileOption) {
                        
                        FileOption fileopt = (FileOption) option;
                        
                        if (fileopt.checkArgument(arg) == false) {
                            buffer += MessageServices.format(res.getString("C_ERROR_OPTION_INVALID_FILE"), 
                                                             option.getUniqueName(), 
                                                             arg)
                                + "\n";
                            
                            argList.remove(index + 1);
                            argList.remove(index);
                            
                            index = searchArgs(argList, option);
                            continue;
                        }
                        
                    } // end of if ()
                    
                    if (option.isLowerCase()) {
                        arg = arg.toLowerCase();
                    } // end of if (option.isIgnoreCase())
                    
                    if (option.isUnset() || option.isUnique()) {
                        option.setValue(arg);
                    } else {
                        option.addValue(arg);
                    }

                    option.isPresent(true);
                
                    argList.remove(index + 1);
                    argList.remove(index);

                    /* This option can have multiple arguments */
                    if (option.isUnique() == false) {

                        int i;

                        for (i = 0; i < argList.size(); i++) {

                            arg = (String) argList.get(i);

                            if (arg.startsWith("-") == false) {
                                option.addValue(arg);
                            } else {
                                break;
                            }

                        }

                        for (int j = 0; j < i; j++) {
                            argList.remove(0); 
                        }
                    }
                    
                } else {

                    option.setValue("true");
                    option.isPresent(true);
                    argList.remove(index);
                }

                // look for more appearances of this option.
                index = searchArgs(argList, option);

                if (option.isUnique() 
                    && index != -1 
                    && counter == 0) {
                    
                    buffer += MessageServices.format(res.getString("C_ERROR_OPTION_WAS_FOUND_MORE_THAN_ONCE"), 
                                                     option.getUniqueName())
                        + "\n";
                }

                counter++;
                
            } // end of while ()
        } // end of while ()
        
        if (argList.size() > 0 && ignoreUnknownOptions == false) {
            
            str = "";

            for (int i = 0; i<argList.size(); i++) {
                str += (String) argList.get(i) + " ";
            }

            buffer += MessageServices.format(res.getString("C_ERROR_OPTION_UNKNOWN"), 
                                             str) + "\n";
        } // end of if ()
        
        if (buffer != null && buffer.length() > 0) {
            error = buffer;
            return false;
        } else {
            return true;
        }
    }
        
    public void write(PrintWriter writer) {
        
        Option option;
        String name;
        
        Object[] options = optList.toArray();
        Arrays.sort(options);
        
        for (int i = 0; i < options.length; i++) {

            option = (Option) options[i];
            name = option.getPropertyName();

            if (name != null && name.length() > 0) {
                writer.println(option.getPropertyName() + "=" + option.getValue());
            }
        }
    }
    
    /**
     * Describe <code>list</code> method here.
     *
     * @return a <code>String</code> value
     */
    public String list() {
                
        StringBuffer buffer = new StringBuffer();

        Object[] options = optList.toArray();
        Arrays.sort(options);

        for (int i = 0; i < options.length; i++) {
            buffer.append(((Option) options[i]).toString() + "\n");
        }

        return buffer.toString();
        
    }
    
    /**
     * Describe <code>init</code> method here.
     *
     * @param cmdline a <code>String[]</code> value
     */
    private void init(String[] cmdline) {

        if (cmdline == null)
            throw new IllegalArgumentException("Variable args is null!");
        
        args = new ArrayList();

        String arg;
        int pos;

        for (int i = 0; i<cmdline.length; i++) {
            
            arg = cmdline[i];
            
            pos = arg.indexOf('=');

            if (arg.startsWith("--") && pos > 0) {

                args.add(arg.substring(0, pos)); 
                           
                if (pos + 1 < arg.length())
                    args.add(arg.substring(pos + 1));
                else
                    args.add("");

            } else {
                args.add(cmdline[i]);
            }
        }

        optList = new ArrayList();
        optMap = new HashMap();
        deprecatedPropertiesMap = new HashMap();
    }

    /**
     * Describe <code>searchArgs</code> method here.
     *
     * @param longName a <code>String</code> value
     * @param shortName a <code>String</code> value
     * @return an <code>int</code> value
     */
    private int searchArgs(ArrayList args, Option option) {
        
        if (args == null) {
            throw new IllegalArgumentException(" The argument args may not be null!");
        }
 
        if (option == null) {
            throw new IllegalArgumentException(" The argument option may not be null!");
        }
 
        String arg;
        
        for (int i = 0; i<args.size(); i++) {
            
            arg = (String) args.get(i);
            
            if (arg.equals("--" + option.getLongName())
                || arg.equals("-" + option.getMediumName())
                || arg.equals("-" + option.getShortName())) {

                return i;
            } // end of if ()
            
        }
        
        return -1;
    }
    
    /**
     * Describe <code>exists</code> method here.
     *
     * @param longName a <code>String</code> value
     * @param mediumName a <code>String</code> value
     * @param shortName a <code>String</code> value
     * @param propertyName a <code>String</code> value
     * @return a <code>boolean</code> value
     */
    private boolean exists(Option lookFor) {
        
        Option option;

        if (optList.contains(lookFor)) {
            return true;
        } // end of if (option.equals(lookFor))

        if (isOptionNameAvailable(lookFor.getLongName()) == false) {
            return true;
        }

        if (isOptionNameAvailable(lookFor.getMediumName()) == false) {
            return true;
        }

        if (isOptionNameAvailable(lookFor.getShortName()) == false) {
            return true;
        }

        if (isOptionNameAvailable(lookFor.getPropertyName()) == false) {
            return true;
        }

        return false;
    }

    private boolean isOptionNameAvailable(String name) {

        if (name != null 
            && name.length() > 0 
            && optMap.get(name) != null) {
            return false;
        }
        
        return true;
    }

}
/*
 * $Log: OptionList.java, v $
 * Revision 1.1.1.1  2003/10/21 19:19:15  cvs
 * Getopts library.
 *
 * Revision 1.2  2003/08/18 12:39:52  mfuchs
 * Code cleanup amd minor improvements.
 *
 * Revision 1.1  2003/08/13 18:39:47  mfuchs
 * Option
 *
 * Revision 1.1  2003/08/12 19:28:10  mfuchs
 * log4j properties files for differnet logging levels.
 *
 */
